from constants import *
from exceptions import *
from pritunl import app_server
from server import Server
from organization import Organization
from event import Event
from log_entry import LogEntry
from cache import cache_db
import httplib
import logging
import os
import json
import utils
import threading
import time
import websocket

logger = logging.getLogger(APP_NAME)

class NodeServer(Server):
    str_options = Server.str_options | {'node_host', 'node_key'}
    int_options = Server.int_options | {'node_port'}
    default_options = dict(Server.default_options.items() + {
        'node_port': 9800,
    }.items())
    type = NODE_SERVER

    def dict(self):
        server_dict = Server.dict(self)
        server_dict.update({
            'node_host': self.node_host,
            'node_port': self.node_port,
            'node_key': self.node_key,
        })
        return server_dict

    def _initialize(self):
        Server._initialize(self)
        with open(os.path.join(self.path, NODE_SERVER), 'w'):
            pass

    def _request(self, method, endpoint='', json_data=None):
        return getattr(utils.request, method)(
            self._get_node_url(endpoint=endpoint),
            timeout=HTTP_REQUEST_TIMEOUT,
            headers={
                'API-Key': self.node_key,
            },
            json_data=json_data,
        )

    def _get_node_url(self, scheme='https', endpoint=''):
        return '%s://%s:%s/server/%s%s' % (
            scheme, self.node_host, self.node_port, self.id, endpoint)

    def _com_on_message(self, ws, message):
        responses = []
        for call in json.loads(message):
            try:
                responses.append({
                    'id': call['id'],
                    'response': getattr(self, call['command'])(
                        *call['args']),
                })
            except:
                logger.exception('Node server com thread call ' + \
                    'failed. %s' % {
                        'server_id': self.id,
                        'call_id': call['id'],
                        'call_command': call['command'],
                        'call_args': call['args'],
                    })
        ws.send(json.dumps(responses))

    def _com_on_error(self, ws, error):
        if isinstance(error, websocket.WebSocketConnectionClosedException) or \
                not isinstance(error, websocket.WebSocketException):
            return
        logger.exception('Error with node server ' + \
            'connection occurred. %r' % {
                'server_id': self.id,
                'error': error.message,
            })
        LogEntry(message='Error with node server ' + \
            'connection occurred "%s".' % self.name)

    def _start_server_threads(self):
        self._state = True
        self._ws = None

        def com_thread():
            ws = websocket.WebSocketApp(
                self._get_node_url('wss', '/com'),
                header=['API-Key: %s' % self.node_key],
                on_message=self._com_on_message,
                on_error=self._com_on_error,
            )
            self._ws = ws
            ws.run_forever(ping_interval=SOCKET_PING_INTERVAL,
                timeout=SOCKET_TIMEOUT)
            self.status = False
            self.publish('stopped')
            self.update_clients({}, force=True)

            if self._state:
                LogEntry(message='Node server stopped unexpectedly "%s".' % (
                    self.name))
                Event(type=SERVERS_UPDATED)

        def sub_thread():
            for message in cache_db.subscribe(self.get_cache_key()):
                try:
                    if message == 'stop':
                        self._state = False
                        if self._ws:
                            self._ws.close()
                    elif message == 'stopped':
                        break
                except OSError:
                    pass

        thread = threading.Thread(target=com_thread)
        thread.daemon = True
        thread.start()
        thread = threading.Thread(target=sub_thread)
        thread.daemon = True
        thread.start()

    def tls_verify(self, org_id, user_id):
        org = self.get_org(org_id)
        if not org:
            LogEntry(message='User failed authentication, ' +
                'invalid organization "%s".' % server.name)
            return False
        user = org.get_user(user_id)
        if not user:
            LogEntry(message='User failed authentication, ' +
                'invalid user "%s".' % server.name)
            return False
        return True

    def otp_verify(self, org_id, user_id, otp_code, remote_ip=None):
        org = self.get_org(org_id)
        if not org:
            LogEntry(message='User failed authentication, ' +
                'invalid organization "%s".' % server.name)
            return False
        user = org.get_user(user_id)
        if not user:
            LogEntry(message='User failed authentication, ' +
                'invalid user "%s".' % server.name)
            return False
        if not user.verify_otp_code(otp_code, remote_ip):
            LogEntry(message='User failed two-step authentication "%s".' % (
                user.name))
            return False
        return True

    def client_connect(self, org_id, user_id):
        org = self.get_org(org_id)
        if not org:
            LogEntry(message='User failed authentication, ' +
                'invalid organization "%s".' % server.name)
            return
        user = org.get_user(user_id)
        if not user:
            LogEntry(message='User failed authentication, ' +
                'invalid user "%s".' % server.name)
            return
        return

    def client_disconnect(self, org_id, user_id):
        org = self.get_org(org_id)
        if not org:
            LogEntry(message='User failed authentication, ' +
                'invalid organization "%s".' % server.name)
            return
        user = org.get_user(user_id)
        if not user:
            LogEntry(message='User failed authentication, ' +
                'invalid user "%s".' % server.name)
            return
        return

    def _generate_ovpn_conf(self):
        if not self.org_count:
            raise ServerMissingOrg('Ovpn conf cannot be generated without ' + \
                'any organizations', {
                    'server_id': self.id,
                })

        logger.debug('Generating node server ovpn conf. %r' % {
            'server_id': self.id,
        })

        if not self.primary_organization or not self.primary_user:
            self._create_primary_user()

        if not os.path.isfile(self.dh_param_path):
            self._generate_dh_param()

        primary_org = Organization.get_org(id=self.primary_organization)
        if not primary_org:
            self._create_primary_user()
        primary_org = Organization.get_org(id=self.primary_organization)

        primary_user = primary_org.get_user(self.primary_user)
        if not primary_user:
            self._create_primary_user()
        primary_user = primary_org.get_user(self.primary_user)

        self.generate_ca_cert()

        push = ''
        if self.mode == LOCAL_TRAFFIC:
            for network in self.local_networks:
                push += 'push "route %s %s"\n' % self._parse_network(network)
        elif self.mode == VPN_TRAFFIC:
            pass
        else:
            push += 'push "redirect-gateway"\n'
        for dns_server in self.dns_servers:
            push += 'push "dhcp-option DNS %s"\n' % dns_server
        push = push.rstrip()

        server_conf = OVPN_INLINE_SERVER_CONF % (
            self.port,
            self.protocol,
            self.interface,
            '%s',
            '%s',
            '%s',
            '%s %s' % self._parse_network(self.network),
            push,
            '%s',
            4 if self.debug else 1,
            8 if self.debug else 3,
        )

        if self.otp_auth:
            server_conf += 'auth-user-pass-verify ' + \
                '<%= user_pass_verify_path %> via-file\n'

        if self.lzo_compression:
            server_conf += 'comp-lzo\npush "comp-lzo"\n'

        if self.local_networks:
            server_conf += 'client-to-client\n'

        server_conf += '<ca>\n%s\n</ca>\n' % utils.get_cert_block(
            self.ca_cert_path)
        server_conf += '<cert>\n%s\n</cert>\n' % utils.get_cert_block(
            primary_user.cert_path)
        server_conf += '<key>\n%s\n</key>\n' % open(
            primary_user.key_path).read().strip()
        server_conf += '<dh>\n%s\n</dh>\n' % open(
            self.dh_param_path).read().strip()

        return server_conf

    def start(self, silent=False):
        cache_db.lock_acquire(self.get_cache_key('op_lock'))
        try:
            if self.status:
                return
            if not self.org_count:
                raise ServerMissingOrg('Server cannot be started without ' + \
                    'any organizations', {
                        'server_id': self.id,
                    })

            logger.debug('Starting node server. %r' % {
                'server_id': self.id,
            })
            ovpn_conf = self._generate_ovpn_conf()

            try:
                response = self._request('post', json_data={
                    'interface': self.interface,
                    'network': self.network,
                    'local_networks': self.local_networks,
                    'ovpn_conf': ovpn_conf,
                    'server_ver': NODE_SERVER_VER,
                })
            except httplib.HTTPException:
                raise NodeConnectionError('Failed to connect to node server', {
                    'server_id': self.id,
                })

            if response.status_code == 401:
                raise InvalidNodeAPIKey('Invalid node server api key', {
                    'server_id': self.id,
                    'status_code': response.status_code,
                    'reason': response.reason,
                })
            elif response.status_code != 200:
                raise ServerStartError('Failed to start node server', {
                    'server_id': self.id,
                    'status_code': response.status_code,
                    'reason': response.reason,
                })

            cache_db.dict_set(self.get_cache_key(), 'start_time',
                str(int(time.time() - 1)))
            self.clear_output()
            self._interrupt = False
            self.status = True
            self._start_server_threads()

            if not silent:
                Event(type=SERVERS_UPDATED)
                LogEntry(message='Started server "%s".' % self.name)
        finally:
            cache_db.lock_release(self.get_cache_key('op_lock'))

    def stop(self, silent=False):
        cache_db.lock_acquire(self.get_cache_key('op_lock'))
        try:
            if not self.status:
                return

            stopped = False
            self.publish('stop')
            for message in cache_db.subscribe(self.get_cache_key(),
                    SUB_RESPONSE_TIMEOUT):
                if message == 'stopped':
                    stopped = True
                    break
            if not stopped:
                raise ServerStopError('Server thread failed to return ' + \
                    'stop event', {
                        'server_id': self.id,
                    })

            if not silent:
                Event(type=SERVERS_UPDATED)
                LogEntry(message='Stopped server "%s".' % self.name)
        finally:
            cache_db.lock_release(self.get_cache_key('op_lock'))

    def force_stop(self, silent=False):
        self.stop(silent)



from pritunl.constants import *
from pritunl.organization import Organization
from pritunl.server import Server
import pritunl.utils as utils
from pritunl import app_server
import os
import flask
import tarfile
import time

def tar_add(tar_file, path):
    if os.path.exists(path):
        tar_file.add(path, arcname=os.path.relpath(path, app_server.data_path))

@app_server.app.route('/export', methods=['GET'])
@app_server.app.route('/export/%s.tar' % APP_NAME, methods=['GET'])
@app_server.auth
def export_get():
    data_path = app_server.data_path
    temp_path = os.path.join(data_path, TEMP_DIR)
    empty_temp_path = os.path.join(temp_path, EMPTY_TEMP_DIR)
    data_archive_name = '%s_%s.tar' % (APP_NAME,
        time.strftime('%Y_%m_%d_%H_%M_%S', time.localtime()))
    data_archive_path = os.path.join(temp_path, data_archive_name)

    # Create empty temp directory to recreate temp dirs in tarfile
    if not os.path.exists(empty_temp_path):
        os.makedirs(empty_temp_path)

    tar_file = tarfile.open(data_archive_path, 'w')
    try:
        tar_add(tar_file, os.path.join(data_path, AUTH_LOG_NAME))
        tar_add(tar_file, os.path.join(data_path, 'pritunl.db'))
        tar_add(tar_file, os.path.join(data_path, SERVER_CERT_NAME))
        tar_add(tar_file, os.path.join(data_path, SERVER_KEY_NAME))
        tar_add(tar_file, os.path.join(data_path, VERSION_NAME))

        for org in Organization.iter_orgs():
            tar_add(tar_file, org.get_path())
            tar_file.add(empty_temp_path,
                arcname=os.path.relpath(os.path.join(org.path, TEMP_DIR),
                    data_path))

            for user in org.iter_users():
                tar_add(tar_file, user.reqs_path)
                tar_add(tar_file, user.key_path)
                tar_add(tar_file, user.cert_path)
                tar_add(tar_file, user.get_path())

            tar_add(tar_file, org.ca_cert.reqs_path)
            tar_add(tar_file, org.ca_cert.key_path)
            tar_add(tar_file, org.ca_cert.cert_path)
            tar_add(tar_file, org.ca_cert.get_path())

        for server in Server.iter_servers():
            tar_add(tar_file, server.dh_param_path)
            tar_add(tar_file, server.ip_pool_path)
            tar_add(tar_file, server.get_path())
            tar_add(tar_file, os.path.join(server.path, NODE_SERVER))
            tar_file.add(empty_temp_path,
                arcname=os.path.relpath(os.path.join(server.path, TEMP_DIR),
                    data_path))

        tar_file.close()

        with open(data_archive_path, 'r') as archive_file:
            response = flask.Response(response=archive_file.read(),
                mimetype='application/octet-stream')
            response.headers.add('Content-Disposition',
                'attachment; filename="%s"' % data_archive_name)
        return response
    finally:
        try:
            tar_file.close()
        except OSError:
            pass
        try:
            os.remove(data_archive_path)
        except OSError:
            pass



from pritunl.constants import *
from pritunl.exceptions import *
from pritunl.organization import Organization
from pritunl.static_file import StaticFile
import pritunl.utils as utils
from pritunl.cache import cache_db
from pritunl import app_server
import os
import flask
import uuid
import time
import random
import json

def _get_key_archive(org_id, user_id):
    org = Organization.get_org(id=org_id)
    user = org.get_user(user_id)
    archive_temp_path = user.build_key_archive()
    try:
        with open(archive_temp_path, 'r') as archive_file:
            response = flask.Response(response=archive_file.read(),
                mimetype='application/octet-stream')
            response.headers.add('Content-Disposition',
                'attachment; filename="%s.tar"' % user.name)
    finally:
        user.clean_key_archive()
    return response

@app_server.app.route('/key/<org_id>/<user_id>.tar', methods=['GET'])
@app_server.auth
def user_key_archive_get(org_id, user_id):
    return _get_key_archive(org_id, user_id)

@app_server.app.route('/key/<org_id>/<user_id>', methods=['GET'])
@app_server.auth
def user_key_link_get(org_id, user_id):
    org = Organization.get_org(id=org_id)
    return utils.jsonify(org.create_user_key_link(user_id))

@app_server.app.route('/key/<key_id>.tar', methods=['GET'])
def user_linked_key_archive_get(key_id):
    key_id_key = 'key_token-%s' % key_id
    org_id = cache_db.dict_get(key_id_key, 'org_id')
    user_id = cache_db.dict_get(key_id_key, 'user_id')

    # Check for expire
    if not cache_db.exists(key_id_key):
        time.sleep(RATE_LIMIT_SLEEP)
        return flask.abort(404)

    return _get_key_archive(org_id, user_id)

@app_server.app.route('/k/<view_id>', methods=['GET'])
def user_linked_key_page_get(view_id):
    view_id_key = 'view_token-%s' % view_id
    org_id = cache_db.dict_get(view_id_key, 'org_id')
    user_id = cache_db.dict_get(view_id_key, 'user_id')
    key_id = cache_db.dict_get(view_id_key, 'key_id')
    conf_urls = cache_db.dict_get(view_id_key, 'conf_urls')
    if conf_urls:
        conf_urls = json.loads(conf_urls)

    # Check for expire
    if not cache_db.exists(view_id_key):
        time.sleep(RATE_LIMIT_SLEEP)
        return flask.abort(404)

    org = Organization.get_org(id=org_id)
    user = org.get_user(user_id)

    key_page = StaticFile(app_server.www_path, KEY_INDEX_NAME,
        cache=False).data
    key_page = key_page.replace('<%= user_name %>', '%s - %s' % (
        org.name, user.name))
    key_page = key_page.replace('<%= user_key_url %>', '/key/%s.tar' % (
        key_id))

    if org.otp_auth:
        key_page = key_page.replace('<%= user_otp_key %>', user.otp_secret)
        key_page = key_page.replace('<%= user_otp_url %>',
            'otpauth://totp/%s@%s?secret=%s' % (
                user.name, org.name, user.otp_secret))
    else:
        key_page = key_page.replace('<%= user_otp_key %>', '')
        key_page = key_page.replace('<%= user_otp_url %>', '')

    key_page = key_page.replace('<%= view_id %>', view_id)

    conf_links = ''
    for conf_url in conf_urls:
        conf_links += '<a class="sm" title="Download Key" ' + \
            'href="%s">Download Key (%s)</a><br>\n' % (
                conf_url['url'], conf_url['server_name'])
    key_page = key_page.replace('<%= conf_links %>', conf_links)

    return key_page

@app_server.app.route('/k/<view_id>', methods=['DELETE'])
def user_linked_key_page_delete_get(view_id):
    view_id_key = 'view_token-%s' % view_id

    key_id = cache_db.dict_get(view_id_key, 'key_id')
    if key_id:
        cache_db.remove('key_token-%s' % key_id)

    uri_id = cache_db.dict_get(view_id_key, 'uri_id')
    if uri_id:
        cache_db.remove('uri_token-%s' % uri_id)

    conf_urls = cache_db.dict_get(view_id_key, 'conf_urls')
    if conf_urls:
        for conf_url in json.loads(conf_urls):
            cache_db.remove('conf_token-%s' % conf_url['id'])

    cache_db.remove(view_id_key)
    return utils.jsonify({})

@app_server.app.route('/ku/<uri_id>', methods=['GET'])
def user_uri_key_page_get(uri_id):
    uri_id_key = 'uri_token-%s' % uri_id
    org_id = cache_db.dict_get(uri_id_key, 'org_id')
    user_id = cache_db.dict_get(uri_id_key, 'user_id')

    # Check for expire
    if not cache_db.exists(uri_id_key):
        time.sleep(RATE_LIMIT_SLEEP)
        return flask.abort(404)

    org = Organization.get_org(id=org_id)
    user = org.get_user(user_id)

    keys = {}
    for server in org.iter_servers():
        key = user.build_key_conf(server.id)
        keys[key['name']] = key['conf']

    return utils.jsonify(keys)

@app_server.app.route('/key/<conf_id>.ovpn', methods=['GET'])
def user_linked_key_conf_get(conf_id):
    conf_id_key = 'conf_token-%s' % conf_id
    org_id = cache_db.dict_get(conf_id_key, 'org_id')
    user_id = cache_db.dict_get(conf_id_key, 'user_id')
    server_id = cache_db.dict_get(conf_id_key, 'server_id')

    # Check for expire
    if not cache_db.exists(conf_id_key):
        time.sleep(RATE_LIMIT_SLEEP)
        return flask.abort(404)

    org = Organization.get_org(id=org_id)
    user = org.get_user(user_id)
    key_conf = user.build_key_conf(server_id)

    response = flask.Response(response=key_conf['conf'],
        mimetype='application/octet-stream')
    response.headers.add('Content-Disposition',
        'attachment; filename="%s"' % key_conf['name'])

    return response



from pritunl.constants import *
import pritunl.utils as utils
from pritunl import app_server
import flask
import re

def _is_vpn_path(path):
    if re.match(r'^/server/[a-z0-9]+/tls_verify$', path) or \
            re.match(r'^/server/[a-z0-9]+/otp_verify$', path) or \
            re.match(r'^/server/[a-z0-9]+/client_connect$', path) or \
            re.match(r'^/server/[a-z0-9]+/client_disconnect$', path):
        return True
    return False

@app_server.app.before_request
def before_request():
    if app_server.www_state == DISABLED and \
            not _is_vpn_path(flask.request.path):
        raise flask.abort(401, app_server.notification)
    elif app_server.vpn_state == DISABLED and _is_vpn_path(flask.request.path):
        raise flask.abort(401, app_server.notification)



from constants import *
from exceptions import *
from pritunl import app_server
from cache import cache_db, persist_db
from organization import Organization
import logging
import time
import threading
import uuid
import subprocess
import os
import itertools

logger = logging.getLogger(APP_NAME)

class Pooler:
    def __init__(self):
        self._orgs_users_thread = None
        self._servers_thread = None

        self.dh_pool_path = os.path.join(app_server.data_path, DH_POOL_DIR)
        if not os.path.exists(self.dh_pool_path):
            os.makedirs(self.dh_pool_path)

        for dh_param in os.listdir(self.dh_pool_path):
            dh_param_path = os.path.join(self.dh_pool_path, dh_param)

            dh_param_split = dh_param.split('_')
            if len(dh_param_split) != 2:
                logger.warning(
                    'Invalid dh param name in pool, skipping... %r' % {
                        'path': dh_param_path
                    })
                continue

            dh_param_bits, dh_param_id = dh_param_split
            try:
                dh_param_bits = int(dh_param_bits)
            except ValueError:
                logger.warning(
                    'Invalid dh param size in pool, skipping... %r' % {
                        'path': dh_param_path
                    })
                continue

            if dh_param_bits not in VALID_DH_PARAM_BITS:
                logger.warning(
                    'Unknown dh param size in pool, skipping... %r' % {
                        'path': dh_param_path
                    })
                continue

            if len(dh_param_id) != 32 or not dh_param_id.isalnum() or \
                    not dh_param_id.islower():
                logger.warning(
                    'Invalid dh param id in pool, skipping... %r' % {
                        'path': dh_param_path
                    })
                continue

            cache_db.dict_set('dh_pool_state', dh_param_path, COMPLETE)
            cache_db.set_add('dh_pool_%s' % dh_param_bits, dh_param_path)

    def _fill_orgs_pool(self):
        for _ in xrange(app_server.org_pool_size -
                Organization.get_org_pool_count()):
            Organization(pool=True)

    def _fill_users_pool(self):
        end = True
        for org in itertools.chain(Organization.iter_orgs(),
                Organization.iter_orgs_pool()):
            if not cache_db.set_length('openssl_tasks') and \
                    app_server.user_pool_size - org.client_pool_count > 0:
                end = False
                org.new_user(type=CERT_CLIENT_POOL)
            if not cache_db.set_length('openssl_tasks') and \
                    app_server.server_user_pool_size - \
                    org.server_pool_size > 0:
                end = False
                org.new_user(type=CERT_SERVER_POOL)
        if not end:
            self._fill_users_pool()

    def _fill_orgs_users(self):
        retry = False
        try:
            self._fill_orgs_pool()
        except:
            logger.exception('Error filling orgs pool, retying...')
            retry = True

        try:
            self._fill_users_pool()
        except:
            # Exception can occur when org is deleted whiling filling
            logger.debug('Error filling users pool, retying...')
            retry = True

        if retry:
            time.sleep(3)
            self._fill_orgs_users()

    def _fill_orgs_users_thread(self):
        self._fill_orgs_users()

        for msg in cache_db.subscribe('pooler'):
            if msg == 'update':
                time.sleep(1)
                self._fill_orgs_users()
            elif msg == 'stop':
                return

    def _fill_servers_pool_listener(self, cache_key, dh_param_path, process):
        for msg in cache_db.subscribe('pooler'):
            if msg == 'update':
                if process.poll() is not None:
                    break
                elif cache_db.set_length('openssl_tasks') and \
                        cache_db.set_exists(cache_key, dh_param_path):
                    # There is a running openssl process started by the user
                    # which takes priority over the pooler's dhparam process
                    if process.poll() is None:
                        process.terminate()
                        time.sleep(0.3)
                        if process.poll() is None:
                            process.kill()
            elif msg == 'stop':
                process.kill()
                return

    def _gen_dh_params(self, dh_param_bits):
        exit_code = None
        cache_key = 'dh_pool_%s' % dh_param_bits
        dh_param_path = os.path.join(self.dh_pool_path,
            '%s_%s' % (dh_param_bits, uuid.uuid4().hex))
        try:
            cache_db.dict_set('dh_pool_state', dh_param_path, PENDING)
            cache_db.set_add(cache_key, dh_param_path)

            args = [
                'openssl', 'dhparam',
                '-out', dh_param_path,
                dh_param_bits,
            ]
            process = subprocess.Popen(args, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)

            listener_thread = threading.Thread(
                target=self._fill_servers_pool_listener,
                args=(cache_key, dh_param_path, process),
            )
            listener_thread.daemon = True
            listener_thread.start()

            exit_code = process.wait()
        finally:
            if exit_code == 0:
                cache_db.dict_set('dh_pool_state', dh_param_path, COMPLETE)
            else:
                cache_db.set_remove(cache_key, dh_param_path)
                cache_db.dict_remove('dh_pool_state', dh_param_path)

                try:
                    os.remove(dh_param_path)
                except:
                    pass

                logger.error('Openssl dhparam returned ' + \
                    'error exit code %r.' % exit_code)
            cache_db.publish('pooler', 'update')

    def _fill_servers_pool(self):
        end = True
        for dh_param_bits in app_server.dh_param_bits_pool:
            dh_pool_size = cache_db.set_length('dh_pool_%s' % dh_param_bits)

            if app_server.server_pool_size - dh_pool_size and not \
                    cache_db.set_length('openssl_tasks'):
                end = False
                self._gen_dh_params(dh_param_bits)
        if not end:
            self._fill_servers_pool()

    def _fill_servers(self):
        try:
            self._fill_servers_pool()
        except:
            logger.exception('Error filling servers pool, retying...')
            time.sleep(3)
            self._fill_servers()

    def _fill_servers_thread(self):
        self._fill_servers()

        for msg in cache_db.subscribe('pooler'):
            if msg == 'update':
                time.sleep(2)
                self._fill_servers()
            elif msg == 'stop':
                return

    def start(self):
        if self._orgs_users_thread or self._servers_thread:
            return
        self._orgs_users_thread = threading.Thread(
            target=self._fill_orgs_users_thread)
        self._orgs_users_thread.daemon = True
        self._orgs_users_thread.start()
        self._servers_thread = threading.Thread(
            target=self._fill_servers_thread)
        self._servers_thread.daemon = True
        self._servers_thread.start()

    def stop(self):
        cache_db.publish('pooler', 'stop')



from constants import *
import Queue
import time
import collections
import threading
import thread
import uuid
import copy
import itertools
import json
import os

TRANSACTION_METHODS = {
    'set',
    'increment',
    'decrement',
    'remove',
    'rename',
    'expire',
    'set_add',
    'set_remove',
    'set_pop',
    'list_lpush',
    'list_rpush',
    'list_lpop',
    'list_rpop',
    'list_remove',
    'dict_set',
    'dict_remove',
}
CHANNEL_TTL = 120
CHANNEL_BUFFER = 128

class TunlDB:
    def __init__(self):
        self._path = None
        self._set_queue = Queue.Queue()
        self._data = collections.defaultdict(
            lambda: {'ttl': None, 'val': None})
        self._timers = {}
        self._channels = collections.defaultdict(
            lambda: {'subs': set(), 'msgs': collections.deque(
                maxlen=CHANNEL_BUFFER), 'timer': None})
        self._commit_log = []
        self._locks = collections.defaultdict(lambda: threading.Lock())

    def _put_queue(self):
        if self._path:
            self._set_queue.put('set')

    def _export_thread(self):
        while True:
            try:
                self._set_queue.get(timeout=5)
            except Queue.Empty:
                continue
            # Attempt to get more db sets form queue to reduce export calls
            for i in xrange(50):
                try:
                    self._set_queue.get(timeout=0.01)
                except Queue.Empty:
                    pass
            self.export_data()

    def _validate(self, value):
        if value is not None and not isinstance(value, basestring):
            raise TypeError('Value must be string')

    def persist(self, path, auto_export=True):
        if self._path:
            raise ValueError('Persist is already set')
        self._path = path
        self.import_data()
        if auto_export:
            thread = threading.Thread(target=self._export_thread)
            thread.daemon = True
            thread.start()

    def set(self, key, value):
        self._validate(value)
        self._data[key]['val'] = value
        self._put_queue()

    def get(self, key):
        data = self._data.get(key)
        if data:
            return data['val']

    def exists(self, key):
        return key in self._data

    def rename(self, key, new_key):
        data = self._data.get(key)
        if data:
            self._data[new_key]['val'] = data['val']
            self.remove(key)
            self._put_queue()

    def remove(self, key):
        self._data.pop(key, None)
        self._put_queue()

    def expire(self, key, ttl):
        ttl_time = int(time.time() * 1000) + (ttl * 1000)

        cur_timer = self._timers.pop(key, None)
        if cur_timer:
            cur_timer.cancel()
        timer = threading.Timer(ttl, self.remove, (key,))
        timer.daemon = True
        self._timers[key] = timer
        timer.start()

        self._data[key]['ttl'] = ttl_time
        self._put_queue()

    def increment(self, key):
        value = '1'
        data = self._data.get(key)
        if data:
            try:
                value = str(int(data['val']) + 1)
                data['val'] = value
            except (TypeError, ValueError):
                data['val'] = value
        else:
            self._data[key]['val'] = value
        self._put_queue()
        return value

    def decrement(self, key):
        value = '-1'
        data = self._data.get(key)
        if data:
            try:
                value = str(int(data['val']) - 1)
                data['val'] = value
            except (TypeError, ValueError):
                data['val'] = value
        else:
            self._data[key]['val'] = value
        self._put_queue()
        return value

    def keys(self):
        return set(self._data)

    def set_add(self, key, element):
        self._validate(element)
        data = self._data.get(key)
        if data:
            try:
                data['val'].add(element)
            except AttributeError:
                data['val'] = {element}
        else:
            self._data[key]['val'] = {element}
        self._put_queue()

    def set_remove(self, key, element):
        data = self._data.get(key)
        if data:
            try:
                data['val'].remove(element)
                self._put_queue()
            except (KeyError, AttributeError):
                pass

    def set_pop(self, key):
        value = None
        data = self._data.get(key)
        if data:
            try:
                value = data['val'].pop()
                self._put_queue()
            except (KeyError, AttributeError):
                pass
        return value

    def set_exists(self, key, element):
        data = self._data.get(key)
        if data:
            try:
                return element in data['val']
            except (TypeError, AttributeError):
                pass
        return False

    def set_elements(self, key):
        data = self._data.get(key)
        if data:
            try:
                return data['val'].copy()
            except AttributeError:
                pass
        return set()

    def set_iter(self, key):
        data = self._data.get(key)
        if data:
            try:
                for value in data['val'].copy():
                    yield value
            except AttributeError:
                pass

    def set_length(self, key):
        data = self._data.get(key)
        if data:
            try:
                return len(data['val'])
            except TypeError:
                pass
        return 0

    def list_lpush(self, key, value):
        self._validate(value)
        data = self._data.get(key)
        if data:
            try:
                data['val'].appendleft(value)
            except AttributeError:
                data['val'] = collections.deque([value])
        else:
            self._data[key]['val'] = collections.deque([value])
        self._put_queue()

    def list_rpush(self, key, value):
        self._validate(value)
        data = self._data.get(key)
        if data:
            try:
                data['val'].append(value)
            except AttributeError:
                data['val'] = collections.deque([value])
        else:
            self._data[key]['val'] = collections.deque([value])
        self._put_queue()

    def list_lpop(self, key):
        value = None
        data = self._data.get(key)
        if data:
            try:
                value = data['val'].popleft()
                self._put_queue()
            except (AttributeError, IndexError):
                pass
        return value

    def list_rpop(self, key):
        value = None
        data = self._data.get(key)
        if data:
            try:
                value = data['val'].pop()
                self._put_queue()
            except (AttributeError, IndexError):
                pass
        return value

    def list_index(self, key, index):
        data = self._data.get(key)
        if data:
            try:
                return data['val'][index]
            except (AttributeError, IndexError):
                pass

    def list_elements(self, key):
        data = self._data.get(key)
        if data:
            try:
                return list(data['val'])
            except TypeError:
                pass
        return []

    def list_iter(self, key):
        data = self._data.get(key)
        if data:
            try:
                for value in copy.copy(data['val']):
                    yield value
            except TypeError:
                pass

    def list_iter_range(self, key, start, stop=None):
        data = self._data.get(key)
        if data:
            try:
                for value in itertools.islice(
                        copy.copy(data['val']), start, stop):
                    yield value
            except TypeError:
                pass

    def list_remove(self, key, value, count=1):
        self._validate(value)
        data = self._data.get(key)
        if data:
            if count:
                try:
                    [data['val'].remove(value) for i in xrange(count)]
                except (AttributeError, ValueError):
                    pass
            else:
                try:
                    while True:
                        data['val'].remove(value)
                except (AttributeError, ValueError):
                    pass
            self._put_queue()

    def list_length(self, key):
        data = self._data.get(key)
        if data:
            try:
                return len(data['val'])
            except TypeError:
                pass
        return 0

    def dict_set(self, key, field, value):
        self._validate(value)
        data = self._data.get(key)
        if data:
            try:
                data['val'][field] = value
            except TypeError:
                data['val'] = {field: value}
        else:
            self._data[key]['val'] = {field: value}
        self._put_queue()

    def dict_get(self, key, field):
        data = self._data.get(key)
        if data:
            try:
                return data['val'].get(field)
            except TypeError:
                pass

    def dict_remove(self, key, field):
        data = self._data.get(key)
        if data:
            try:
                data['val'].pop(field, None)
            except AttributeError:
                pass
            self._put_queue()

    def dict_keys(self, key):
        data = self._data.get(key)
        if data:
            try:
                return set(data['val'])
            except AttributeError:
                pass
        return set()

    def dict_values(self, key):
        data = self._data.get(key)
        if data:
            try:
                return set(data['val'].values())
            except AttributeError:
                pass
        return set()

    def dict_iter(self, key):
        data = self._data.get(key)
        if data:
            data_copy = data['val'].copy()
            try:
                for field in data_copy:
                    yield field, data_copy[field]
            except (TypeError, AttributeError):
                pass

    def dict_get_all(self, key):
        data = self._data.get(key)
        if data:
            try:
                return data['val'].copy()
            except AttributeError:
                pass
        return {}

    def _clear_channel(self, channel):
        if not self._channels[channel]['subs']:
            self._channels.pop(channel, None)
        else:
            self._channels[channel]['timer'] = None
            self._channels[channel]['msgs'] = collections.deque(
                maxlen=CHANNEL_BUFFER)

    def subscribe(self, channel, timeout=None):
        event = threading.Event()
        self._channels[channel]['subs'].add(event)
        try:
            try:
                cursor = self._channels[channel]['msgs'][-1][0]
            except IndexError:
                cursor = None
            while True:
                if not cursor:
                    cursor_found = True
                else:
                    cursor_found = False
                if not event.wait(timeout):
                    break
                event.clear()
                messages = copy.copy(self._channels[channel]['msgs'])
                for message in messages:
                    if cursor_found:
                        yield message[1]
                    elif message[0] == cursor:
                        cursor_found = True
                if not cursor_found:
                    for message in messages:
                        yield message[1]
                try:
                    cursor = messages[-1][0]
                except IndexError:
                    cursor = None
        finally:
            try:
                self._channels[channel]['subs'].remove(event)
            except KeyError:
                pass

    def publish(self, channel, message):
        cur_timer = self._channels[channel]['timer']
        if cur_timer:
            cur_timer.cancel()
        timer = threading.Timer(CHANNEL_TTL, self._clear_channel, (channel,))
        timer.daemon = True
        self._channels[channel]['timer'] = timer
        timer.start()

        self._channels[channel]['msgs'].append((uuid.uuid4().hex, message))
        for subscriber in self._channels[channel]['subs'].copy():
            subscriber.set()

    def transaction(self):
        return TunlDBTransaction(self)

    def _apply_trans(self, trans):
        for call in trans[1]:
            getattr(self, call[0])(*call[1], **call[2])
        try:
            self._commit_log.remove(trans)
        except ValueError:
            pass
        self._put_queue()

    def lock_acquire(self, key):
        return self._locks[key].acquire()

    def lock_release(self, key):
        try:
            self._locks[key].release()
        except thread.error:
            pass

    def lock_remove(self, key):
        self.lock_release(key)
        self._locks.pop(key, None)
        return TunlDBTransaction(self)

    def export_data(self):
        if not self._path:
            return
        temp_path = self._path + '_%s.tmp' % uuid.uuid4().hex
        try:
            data = self._data.copy()
            timers = self._timers.keys()
            commit_log = copy.copy(self._commit_log)

            with open(temp_path, 'w') as db_file:
                os.chmod(temp_path, 0600)
                export_data = []

                for key in data:
                    key_ttl = data[key]['ttl']
                    key_val = data[key]['val']
                    key_type = type(key_val).__name__
                    if key_type == 'set' or key_type == 'deque':
                        key_val = list(key_val)
                    export_data.append((key, key_type, key_ttl, key_val))

                db_file.write(json.dumps({
                    'ver': 1,
                    'data': export_data,
                    'timers': timers,
                    'commit_log': commit_log,
                }))
            os.rename(temp_path, self._path)
        except:
            try:
                os.remove(temp_path)
            except OSError:
                pass
            raise

    def import_data(self):
        if os.path.isfile(self._path):
            with open(self._path, 'r') as db_file:
                import_data = json.loads(db_file.read())
                data = import_data['data']

                for key_data in data:
                    key = key_data[0]
                    key_type = key_data[1]
                    key_ttl = key_data[2]
                    key_val = key_data[3]

                    if key_type == 'set':
                        key_val = set(key_val)
                    elif key_type == 'deque':
                        key_val = collections.deque(key_val)

                    self._data[key] = {
                        'ttl': key_ttl,
                        'val': key_val,
                    }

                if 'timers' in import_data:
                    for key in import_data['timers']:
                        if key not in self._data:
                            continue
                        ttl = self._data[key]['ttl']
                        if not ttl:
                            continue
                        ttl -= int(time.time() * 1000)
                        ttl = ttl / 1000.0
                        if ttl >= 0:
                            timer = threading.Timer(ttl, self.remove, (key,))
                            timer.daemon = True
                            self._timers[key] = timer
                            timer.start()
                        else:
                            self.remove(key)

                if 'commit_log' in import_data:
                    for tran in import_data['commit_log']:
                        self._apply_trans(tran)

class TunlDBTransaction:
    def __init__(self, cache):
        self._cache = cache
        self._trans = []

    def __getattr__(self, name):
        if name in TRANSACTION_METHODS:
            def serialize(*args, **kwargs):
                self._trans.append((name, args, kwargs))
            return serialize
        return getattr(self._cache, name)

    def commit(self):
        trans = (uuid.uuid4().hex, self._trans)
        self._cache._commit_log.append(trans)
        self._trans = []
        self._cache._apply_trans(trans)

cache_db = TunlDB()
persist_db = TunlDB()



import threading
import unittest
import requests
import json
import time
import os

BASE_URL = os.getenv('BASE_URL', 'http://localhost:9700')
HEADERS = {
    'Accept': 'application/json',
}
USERNAME = os.getenv('USERNAME', 'admin')
PASSWORD = os.getenv('PASSWORD', 'admin')
TEST_PASSWORD = 'unittest'
TEST_USER_NAME = 'unittest_user'
TEST_ORG_NAME = 'unittest_org'
TEST_SERVER_NAME = 'unittest_server'
TEST_EMAIL_FROM = 'test_email_from'
TEST_EMAIL_API_KEY = 'test_email_api_key'
ENABLE_STANDARD_TESTS = True
ENABLE_EXTENDED_TESTS = False
ENABLE_STRESS_TESTS = False
THREADED_STRESS_TEST = True
UUID_RE = r'^[a-z0-9]+$'
AUTH_HANDLERS = [
    ('PUT', '/auth'),
    ('GET', '/auth'),
    ('GET', '/export'),
    ('GET', '/event'),
    ('GET', '/key/a1/a1.tar'),
    ('GET', '/key/a1/a1'),
    ('GET', '/log'),
    ('GET', '/organization'),
    ('GET', '/organization/a1'),
    ('POST', '/organization'),
    ('PUT', '/organization/a1'),
    ('DELETE', '/organization/a1'),
    ('GET', '/server'),
    ('GET', '/server/a1'),
    ('POST', '/server'),
    ('PUT', '/server/a1'),
    ('DELETE', '/server/a1'),
    ('GET', '/server/a1/organization'),
    ('PUT', '/server/a1/organization/a1'),
    ('DELETE', '/server/a1/organization/a1'),
    ('PUT', '/server/a1/a1'),
    ('GET', '/server/a1/output'),
    ('DELETE', '/server/a1/output'),
    ('GET', '/server/a1/bandwidth'),
    ('GET', '/server/a1/bandwidth/1m'),
    ('GET', '/status'),
    ('GET', '/user/a1'),
    ('GET', '/user/a1/1'),
    ('GET', '/user/a1/a1'),
    ('POST', '/user/a1'),
    ('PUT', '/user/a1/a1'),
    ('DELETE', '/user/a1/a1'),
    ('PUT', '/user/a1/a1/otp_secret'),
]
RUN_ONLY = []

def _log_request(method, endpoint, start_time):
    if endpoint.startswith('/event'):
        return
    response_time = int((time.time() - start_time) * 1000)
    if endpoint.startswith('/auth') or endpoint.startswith('/key/') \
            or endpoint.startswith('/k/') or endpoint.startswith('/ku/'):
        if response_time > 900:
            color = '\033[92m'
        elif response_time > 700:
            color = '\033[93m'
        else:
            color = '\033[92m'
    else:
        if response_time > 400:
            color = '\033[91m'
        elif response_time > 200:
            color = '\033[93m'
        else:
            color = '\033[92m'
    print '%s%sms:%s:%s\033[0m' % (color, response_time, method, endpoint)

_request = requests.api.request
def request(method, endpoint, **kwargs):
    headers = {
        'Accept': 'application/json',
    }
    if 'headers' in kwargs:
        headers.update(kwargs.pop('headers'))
    if 'json_data' in kwargs and kwargs['json_data']:
        headers['Content-Type'] = 'application/json'
        kwargs['data'] = json.dumps(kwargs.pop('json_data'))
    start_time = time.time()
    response = _request(method, BASE_URL + endpoint, headers=headers,
        verify=False, **kwargs)
    _log_request(method, endpoint, start_time)
    return response
requests.api.request = request


class Session:
    def __init__(self):
        self._session = requests.Session()
        self.response = self.post('/auth/session', json_data={
            'username': USERNAME,
            'password': PASSWORD,
        })

    def _request(self, method, endpoint, **kwargs):
        headers = {
            'Accept': 'application/json',
        }
        if 'headers' in kwargs:
            headers.update(kwargs.pop('headers'))
        if 'json_data' in kwargs and kwargs['json_data']:
            headers['Content-Type'] = 'application/json'
            kwargs['data'] = json.dumps(kwargs.pop('json_data'))
        start_time = time.time()
        response = getattr(self._session, method)(BASE_URL + endpoint,
            headers=headers, verify=False, **kwargs)
        _log_request(method, endpoint, start_time)
        return response

    def get(self, endpoint, **kwargs):
        return self._request('get', endpoint, **kwargs)

    def post(self, endpoint, **kwargs):
        return self._request('post', endpoint, **kwargs)

    def put(self, endpoint, **kwargs):
        return self._request('put', endpoint, **kwargs)

    def delete(self, endpoint, **kwargs):
        return self._request('delete', endpoint, **kwargs)


class SessionTestCase(unittest.TestCase):
    session = Session()

    def setUp(self):
        if RUN_ONLY and self._testMethodName not in RUN_ONLY:
            self.skipTest('ignore')

        self.org_id = None
        self.user_id = None
        self.server_id = None
        self._create_test_data()
        self._clean_test_data()

    def _clean_test_data(self):
        response = self.session.get('/organization')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        for org in data:
            if org['name'] in (TEST_ORG_NAME + '2', TEST_ORG_NAME + '3'):
                response = self.session.delete('/organization/%s' % org['id'])
                self.assertEqual(response.status_code, 200)


        response = self.session.get('/user/%s' % self.org_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        for user in data:
            if user['name'] in (TEST_USER_NAME + '2', TEST_USER_NAME + '3'):
                response = self.session.delete('/user/%s/%s' % (
                    self.org_id, user['id']))
                self.assertEqual(response.status_code, 200)
                time.sleep(0.1)


        response = self.session.get('/server')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        for server in data:
            if server['name'] in (TEST_SERVER_NAME + '2',
                    TEST_SERVER_NAME + '3'):
                response = self.session.delete('/server/%s' % server['id'])
                self.assertEqual(response.status_code, 200)


        response = self.session.get('/server/%s/organization' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        for server_org in data:
            response = self.session.delete('/server/%s/organization/%s' % (
                self.server_id, server_org['id']))
            self.assertEqual(response.status_code, 200)

    def _create_test_data(self):
        if not self.org_id:
            response = self.session.get('/organization')
            self.assertEqual(response.status_code, 200)

            data = response.json()
            for org in data:
                if org['name'] == TEST_ORG_NAME:
                    self.org_id = org['id']


        if not self.org_id:
            response = self.session.post('/organization', json_data={
                'name': TEST_ORG_NAME,
            })
            self.assertEqual(response.status_code, 200)

            data = response.json()
            self.assertIn('id', data)
            self.assertIn('name', data)
            self.assertEqual(data['name'], TEST_ORG_NAME)
            self.org_id = data['id']


        if not self.user_id:
            response = self.session.get('/user/%s' % self.org_id)
            self.assertEqual(response.status_code, 200)

            data = response.json()
            for user in data:
                if user['name'] == TEST_USER_NAME:
                    self.user_id = user['id']


        if not self.user_id:
            response = self.session.post('/user/%s' % self.org_id, json_data={
                'name': TEST_USER_NAME,
            })
            self.assertEqual(response.status_code, 200)
            time.sleep(0.1)

            data = response.json()
            self.assertIn('id', data)
            self.assertIn('name', data)
            self.assertEqual(data['name'], TEST_USER_NAME)
            self.user_id = data['id']


        if not self.server_id:
            response = self.session.get('/server')
            self.assertEqual(response.status_code, 200)

            data = response.json()
            for server in data:
                if server['name'] == TEST_SERVER_NAME:
                    self.server_id = server['id']


        if not self.server_id:
            response = self.session.post('/server', json_data={
                'name': TEST_SERVER_NAME,
                'dh_param_bits': 1024,
                'otp_auth': True,
            })
            self.assertEqual(response.status_code, 200)

            data = response.json()
            self.assertIn('id', data)
            self.assertIn('name', data)
            self.assertEqual(data['name'], TEST_SERVER_NAME)
            self.server_id = data['id']

    def _delete_test_data(self):
        response = self.session.delete('/organization/%s' % self.org_id)
        self.assertEqual(response.status_code, 200)


class Auth(SessionTestCase):
    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_auth_put(self):
        response = self.session.put('/auth', json_data={
            'username': USERNAME,
            'password': TEST_PASSWORD,
            'email_from': TEST_EMAIL_FROM,
            'email_api_key': TEST_EMAIL_API_KEY,
        })
        self.assertEqual(response.status_code, 200)


        response = self.session.get('/auth')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('username', data)
        self.assertIn('token', data)
        self.assertIn('secret', data)
        self.assertIn('email_from', data)
        self.assertEqual(data['email_from'], TEST_EMAIL_FROM)
        self.assertIn('email_api_key', data)
        self.assertEqual(data['email_api_key'], TEST_EMAIL_API_KEY)


        response = self.session.put('/auth', json_data={
            'username': USERNAME,
            'password': PASSWORD,
        })
        self.assertEqual(response.status_code, 200)

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_auth_session_get(self):
        response = requests.get('/auth/session')

        data = response.json()
        self.assertEqual(response.status_code, 200)
        self.assertIn('authenticated', data)
        self.assertFalse(data['authenticated'])

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_auth_session_delete(self):
        session = Session()
        data = session.response.json()
        self.assertEqual(session.response.status_code, 200)
        self.assertIn('authenticated', data)
        self.assertTrue(data['authenticated'])


        response = session.get('/auth/session')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('authenticated', data)
        self.assertTrue(data['authenticated'])


        response = session.delete('/auth/session')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('authenticated', data)
        self.assertFalse(data['authenticated'])


        response = session.get('/auth/session')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('authenticated', data)
        self.assertFalse(data['authenticated'])

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_auth_post_error(self):
        for username, password in (
                    ('admin', 'test'),
                    ('test', 'admin'),
                    ('test', 'test'),
                ):
            response = requests.post('/auth/session', json_data={
                'username': username,
                'password': password,
            })
            self.assertEqual(response.status_code, 401)

            data = response.json()
            self.assertIn('error', data)
            self.assertEqual(data['error'], 'auth_invalid')
            self.assertIn('error_msg', data)

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_auth_handlers(self):
        for method, endpoint in AUTH_HANDLERS:
            response = getattr(requests, method.lower())(endpoint)
            self.assertEqual(response.status_code, 401)


class Data(SessionTestCase):
    @unittest.skipUnless(ENABLE_EXTENDED_TESTS, 'Skipping test')
    def test_export_get(self):
        for endpoint in ('/export', '/export/pritunl.tar'):
            response = self.session.get(endpoint)
            self.assertEqual(response.status_code, 200)

            content_type = response.headers['content-type']
            self.assertEqual(content_type, 'application/octet-stream')

            content_disposition = response.headers['content-disposition']
            exp = r'^attachment; filename="pritunl_[0-9]+_[0-9]+_[0-9]+_' + \
                '[0-9]+_[0-9]+_[0-9]+\.tar"$'
            self.assertRegexpMatches(content_disposition, exp)


class Key(SessionTestCase):
    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_user_key_archive_get(self):
        response = self.session.get('/key/%s/%s.tar' % (
            self.org_id, self.user_id))
        self.assertEqual(response.status_code, 200)

        content_type = response.headers['content-type']
        self.assertEqual(content_type, 'application/octet-stream')

        content_disposition = response.headers['content-disposition']
        exp = r'^attachment; filename="%s.tar"$' % TEST_USER_NAME
        self.assertRegexpMatches(content_disposition, exp)

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_user_key_link_get_delete(self):
        response = self.session.put('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 200)


        response = self.session.get('/key/%s/%s' % (
            self.org_id, self.user_id))
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertIn('key_url', data)
        self.assertIn('view_url', data)
        self.assertIn('uri_url', data)

        exp = r'^/key/[a-z0-9]+.tar$'
        self.assertRegexpMatches(data['key_url'], exp)

        exp = r'^/k/[a-zA-Z0-9]+$'
        self.assertRegexpMatches(data['view_url'], exp)

        exp = r'^/ku/[a-zA-Z0-9]+$'
        self.assertRegexpMatches(data['uri_url'], exp)


        response = self.session.get(data['key_url'])
        self.assertEqual(response.status_code, 200)

        content_type = response.headers['content-type']
        self.assertEqual(content_type, 'application/octet-stream')

        content_disposition = response.headers['content-disposition']
        exp = r'^attachment; filename="%s.tar"$' % TEST_USER_NAME
        self.assertRegexpMatches(content_disposition, exp)


        response = self.session.get(data['view_url'])
        self.assertEqual(response.status_code, 200)

        content_type = response.headers['content-type']
        self.assertEqual(content_type, 'text/html; charset=utf-8')

        start_index = response.text.find('<h4 class="key-title">') + 22
        end_index = response.text.find('</h4>', start_index)
        self.assertNotEqual(start_index, -1)
        self.assertNotEqual(end_index, -1)
        key_title = response.text[start_index:end_index]
        self.assertEqual('%s - %s' % (TEST_ORG_NAME, TEST_USER_NAME),
            key_title)

        start_index = response.text.find(
            '<input id="key" type="text" readonly value="') + 44
        end_index = response.text.find('">', start_index)
        self.assertNotEqual(start_index, -1)
        self.assertNotEqual(end_index, -1)
        otp_secret = response.text[start_index:end_index]
        exp = r'^[A-Z0-9]+$'
        self.assertRegexpMatches(otp_secret, exp)

        start_index = response.text.find('<a title="Download Key" href="') + 30
        end_index = response.text.find('">', start_index)
        self.assertNotEqual(start_index, -1)
        self.assertNotEqual(end_index, -1)
        key_url = response.text[start_index:end_index]
        self.assertEqual(key_url, data['key_url'])

        start_index = response.text.find(
            '<a class="sm" title="Download Key" href="') + 41
        end_index = response.text.find('">', start_index)
        self.assertNotEqual(start_index, -1)
        self.assertNotEqual(end_index, -1)
        conf_url = response.text[start_index:end_index]

        exp = r'^/key/[a-z0-9]+\.ovpn$'
        self.assertRegexpMatches(conf_url, exp)

        start_index = response.text.find("text: 'otpauth://totp/") + 7
        end_index = response.text.find("',", start_index)
        self.assertNotEqual(start_index, -1)
        self.assertNotEqual(end_index, -1)
        otp_key = response.text[start_index:end_index]

        exp = r'^otpauth://totp/%s@%s\?secret\=%s$' % (
            TEST_USER_NAME, TEST_ORG_NAME, otp_secret)
        self.assertRegexpMatches(otp_key, exp)

        start_index = response.text.find("xmlhttp.open('DELETE', '") + 24
        end_index = response.text.find("', false);", start_index)
        self.assertNotEqual(start_index, -1)
        self.assertNotEqual(end_index, -1)
        delete_link = response.text[start_index:end_index]
        self.assertEqual(data['view_url'], delete_link)


        response = self.session.get(data['uri_url'])
        self.assertEqual(response.status_code, 200)

        content_type = response.headers['content-type']
        self.assertEqual(content_type, 'application/json')


        response = self.session.delete(data['view_url'])
        self.assertEqual(response.status_code, 200)


        response = self.session.get(data['view_url'])
        self.assertEqual(response.status_code, 404)


        response = self.session.delete('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 200)


class Log(SessionTestCase):
    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_log_get(self):
        response = self.session.get('/log')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertNotEqual(len(data), 0)
        for entry in data:
            self.assertIn('id', entry)
            self.assertRegexpMatches(entry['id'], UUID_RE)
            self.assertIn('time', entry)
            self.assertIn('message', entry)


class Org(SessionTestCase):
    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_org_get(self):
        response = self.session.get('/organization/%s' % self.org_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.org_id)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_ORG_NAME)

    def test_org_post_put_get_delete(self):
        response = self.session.post('/organization', json_data={
            'name': TEST_ORG_NAME + '2',
        })
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertRegexpMatches(data['id'], UUID_RE)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_ORG_NAME + '2')
        org_id = data['id']


        response = self.session.put('/organization/%s' % org_id, json_data={
            'name': TEST_ORG_NAME + '3',
        })
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_ORG_NAME + '3')


        response = self.session.get('/organization')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertNotEqual(len(data), 0)
        test_org_found = False
        for org in data:
            self.assertIn('id', org)
            self.assertRegexpMatches(org['id'], UUID_RE)
            self.assertIn('name', org)
            if org['name'] == TEST_ORG_NAME + '3':
                test_org_found = True
                self.assertEqual(org['id'], org_id)
        self.assertTrue(test_org_found)


        response = self.session.delete('/organization/%s' % org_id)
        self.assertEqual(response.status_code, 200)


        response = self.session.get('/organization')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertNotEqual(len(data), 0)
        for org in data:
            self.assertIn('id', org)
            self.assertIn('name', org)
            self.assertNotEqual(org['name'], TEST_ORG_NAME + '3')


class Server(SessionTestCase):
    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_server_get(self):
        response = self.session.get('/server/%s' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.server_id)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_SERVER_NAME)

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_server_post_put_get_delete(self):
        response = self.session.post('/server', json_data={
            'name': TEST_SERVER_NAME + '2',
            'network': '10.254.254.0/24',
            'interface': 'tun64',
            'port': 12345,
            'protocol': 'udp',
            'dh_param_bits': 1024,
            'mode': 'local_traffic',
            'local_networks': ['10.0.0.0/24'],
            'dns_servers': ['8.8.4.4'],
            'search_domain': 'pritunl.com',
            'public_address': '8.8.8.8',
            'debug': True,
            'otp_auth': False,
            'lzo_compression': False,
        })
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertRegexpMatches(data['id'], UUID_RE)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_SERVER_NAME + '2')
        self.assertIn('network', data)
        self.assertEqual(data['network'], '10.254.254.0/24')
        self.assertIn('interface', data)
        self.assertEqual(data['interface'], 'tun64')
        self.assertIn('port', data)
        self.assertEqual(data['port'], 12345)
        self.assertIn('protocol', data)
        self.assertEqual(data['protocol'], 'udp')
        self.assertIn('dh_param_bits', data)
        self.assertEqual(data['dh_param_bits'], 1024)
        self.assertIn('mode', data)
        self.assertEqual(data['mode'], 'local_traffic')
        self.assertIn('local_networks', data)
        self.assertEqual(data['local_networks'], ['10.0.0.0/24'])
        self.assertIn('dns_servers', data)
        self.assertEqual(data['dns_servers'], ['8.8.4.4'])
        self.assertIn('search_domain', data)
        self.assertEqual(data['search_domain'], 'pritunl.com')
        self.assertIn('public_address', data)
        self.assertEqual(data['public_address'], '8.8.8.8')
        self.assertIn('debug', data)
        self.assertTrue(data['debug'])
        self.assertIn('otp_auth', data)
        self.assertFalse(data['otp_auth'])
        self.assertIn('lzo_compression', data)
        self.assertFalse(data['lzo_compression'])
        server_id = data['id']


        response = self.session.put('/server/%s' % server_id, json_data={
            'name': TEST_SERVER_NAME + '3',
        })
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], server_id)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_SERVER_NAME + '3')
        self.assertIn('network', data)
        self.assertEqual(data['network'], '10.254.254.0/24')
        self.assertIn('interface', data)
        self.assertEqual(data['interface'], 'tun64')
        self.assertIn('port', data)
        self.assertEqual(data['port'], 12345)
        self.assertIn('protocol', data)
        self.assertEqual(data['protocol'], 'udp')
        self.assertIn('dh_param_bits', data)
        self.assertEqual(data['dh_param_bits'], 1024)
        self.assertIn('mode', data)
        self.assertEqual(data['mode'], 'local_traffic')
        self.assertIn('local_networks', data)
        self.assertEqual(data['local_networks'], ['10.0.0.0/24'])
        self.assertIn('dns_servers', data)
        self.assertEqual(data['dns_servers'], ['8.8.4.4'])
        self.assertIn('public_address', data)
        self.assertEqual(data['public_address'], '8.8.8.8')
        self.assertIn('search_domain', data)
        self.assertEqual(data['search_domain'], 'pritunl.com')
        self.assertIn('debug', data)
        self.assertTrue(data['debug'])
        self.assertIn('otp_auth', data)
        self.assertFalse(data['otp_auth'])
        self.assertIn('lzo_compression', data)
        self.assertFalse(data['lzo_compression'])


        response = self.session.get('/server')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertNotEqual(len(data), 0)
        test_server_found = False
        for server in data:
            self.assertIn('id', server)
            self.assertRegexpMatches(server['id'], UUID_RE)
            self.assertIn('name', server)
            self.assertIn('network', server)
            self.assertIn('interface', server)
            self.assertIn('port', server)
            self.assertIn('protocol', server)
            self.assertIn('dh_param_bits', server)
            self.assertIn('mode', server)
            self.assertIn('local_networks', server)
            self.assertIn('dns_servers', server)
            self.assertIn('search_domain', server)
            self.assertIn('public_address', server)
            self.assertIn('debug', server)
            self.assertIn('otp_auth', server)
            self.assertIn('lzo_compression', server)

            if server['name'] == TEST_SERVER_NAME + '3':
                test_server_found = True
                self.assertEqual(server['id'], server_id)
                self.assertEqual(server['network'], '10.254.254.0/24')
                self.assertEqual(server['interface'], 'tun64')
                self.assertEqual(server['port'], 12345)
                self.assertEqual(server['protocol'], 'udp')
                self.assertEqual(server['dh_param_bits'], 1024)
                self.assertEqual(server['mode'], 'local_traffic')
                self.assertEqual(server['local_networks'], ['10.0.0.0/24'])
                self.assertEqual(server['dns_servers'], ['8.8.4.4'])
                self.assertEqual(server['search_domain'], 'pritunl.com')
                self.assertEqual(server['public_address'], '8.8.8.8')
                self.assertTrue(server['debug'])
                self.assertFalse(server['otp_auth'])
                self.assertFalse(server['lzo_compression'])
        self.assertTrue(test_server_found)


        response = self.session.delete('/server/%s' % server_id)
        self.assertEqual(response.status_code, 200)


        response = self.session.get('/server')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertNotEqual(len(data), 0)
        for server in data:
            self.assertIn('id', server)
            self.assertIn('name', server)
            self.assertNotEqual(server['name'], TEST_SERVER_NAME + '3')

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_server_org_put_get_delete(self):
        response = self.session.put('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.org_id)
        self.assertIn('server', data)
        self.assertEqual(data['server'], self.server_id)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_ORG_NAME)


        response = self.session.get('/server/%s/organization' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertNotEqual(len(data), 0)
        test_server_org_found = False
        for server_org in data:
            self.assertIn('id', server_org)
            self.assertRegexpMatches(server_org['id'], UUID_RE)
            self.assertIn('server', server_org)
            self.assertEqual(server_org['server'], self.server_id)
            self.assertIn('name', server_org)
            if server_org['name'] == TEST_ORG_NAME:
                test_server_org_found = True
                self.assertEqual(server_org['id'], self.org_id)
        self.assertTrue(test_server_org_found)


        response = self.session.delete('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 200)

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_server_operation_put(self):
        response = self.session.put('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.org_id)
        self.assertIn('server', data)
        self.assertEqual(data['server'], self.server_id)


        response = self.session.put('/server/%s/start' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.server_id)
        self.assertIn('status', data)
        self.assertTrue(data['status'])


        response = self.session.put('/server/%s/restart' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.server_id)
        self.assertIn('status', data)
        self.assertTrue(data['status'])


        response = self.session.put('/server/%s/stop' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.server_id)
        self.assertIn('status', data)
        self.assertFalse(data['status'])


        response = self.session.delete('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 200)

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_server_output_delete_get(self):
        response = self.session.delete('/server/%s/output' % self.server_id)
        self.assertEqual(response.status_code, 200)

        response = self.session.get('/server/%s/output' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('output', data)
        self.assertEqual(data['output'], '')

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_server_bandwidth_get(self):
        lengths = {
            '1m': 360,
            '5m': 288,
            '30m': 336,
            '2h': 360,
            '1d': 365,
        }

        response = self.session.get('/server/%s/bandwidth' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('1m', data)
        self.assertIn('received', data['1m'])
        self.assertEqual(len(data['1m']['received']), lengths['1m'])
        self.assertIn('sent', data['1m'])
        self.assertEqual(len(data['1m']['sent']), lengths['1m'])
        self.assertIn('5m', data)
        self.assertIn('received', data['5m'])
        self.assertEqual(len(data['5m']['received']), lengths['5m'])
        self.assertIn('sent', data['5m'])
        self.assertEqual(len(data['5m']['sent']), lengths['5m'])
        self.assertIn('30m', data)
        self.assertIn('received', data['30m'])
        self.assertEqual(len(data['30m']['received']), lengths['30m'])
        self.assertIn('sent', data['30m'])
        self.assertEqual(len(data['30m']['sent']), lengths['30m'])
        self.assertIn('2h', data)
        self.assertIn('received', data['2h'])
        self.assertEqual(len(data['2h']['received']), lengths['2h'])
        self.assertIn('sent', data['2h'])
        self.assertEqual(len(data['2h']['sent']), lengths['2h'])
        self.assertIn('1d', data)
        self.assertIn('received', data['1d'])
        self.assertEqual(len(data['1d']['received']), lengths['1d'])
        self.assertIn('sent', data['1d'])
        self.assertEqual(len(data['1d']['sent']), lengths['1d'])


        for period in ('1m', '5m', '30m', '2h', '1d'):
            response = self.session.get('/server/%s/bandwidth/%s' % (
                self.server_id, period))
            self.assertEqual(response.status_code, 200)

            data = response.json()
            self.assertIn('received', data)
            self.assertEqual(len(data['received']), lengths[period])
            self.assertIn('sent', data)
            self.assertEqual(len(data['sent']), lengths[period])

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_server_put_post_errors(self):
        response = self.session.put('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.org_id)
        self.assertIn('server', data)
        self.assertEqual(data['server'], self.server_id)


        response = self.session.put('/server/%s/start' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.server_id)
        self.assertIn('status', data)
        self.assertTrue(data['status'])


        response = self.session.put('/server/%s' % self.server_id, json_data={
            'name': TEST_SERVER_NAME + '_test',
        })
        self.assertEqual(response.status_code, 400)

        data = response.json()
        self.assertIn('error', data)
        self.assertEqual(data['error'], 'server_not_offline')
        self.assertIn('error_msg', data)


        response = self.session.put('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 400)

        data = response.json()
        self.assertIn('error', data)
        self.assertEqual(data['error'], 'server_not_offline')
        self.assertIn('error_msg', data)


        response = self.session.delete('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 400)

        data = response.json()
        self.assertIn('error', data)
        self.assertEqual(data['error'], 'server_not_offline')
        self.assertIn('error_msg', data)


        response = self.session.put('/server/%s/stop' % self.server_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.server_id)
        self.assertIn('status', data)
        self.assertFalse(data['status'])


        response = self.session.delete('/server/%s/organization/%s' % (
            self.server_id, self.org_id))
        self.assertEqual(response.status_code, 200)


        for test_network in (
                    '10.254.254.024',
                    '10254.254.0/24',
                    '10a.254.254.0/24',
                    '11.254.254.0/24',
                    '10.255.254.1/24',
                    '10.254.254.0/24a',
                ):
            response = self.session.post('/server', json_data={
                'name': TEST_SERVER_NAME + '_test',
                'network': test_network,
            })
            self.assertEqual(response.status_code, 400)

            data = response.json()
            self.assertIn('error', data)
            self.assertEqual(data['error'], 'network_invalid')
            self.assertIn('error_msg', data)


        for test_interface in (
                    'tun-1',
                    'tun.0',
                    'tun65',
                    'tuna',
                ):
            response = self.session.post('/server', json_data={
                'name': TEST_SERVER_NAME + '_test',
                'interface': test_interface,
            })
            self.assertEqual(response.status_code, 400)

            data = response.json()
            self.assertIn('error', data)
            self.assertEqual(data['error'], 'interface_invalid')
            self.assertIn('error_msg', data)


        for test_port in (0, 65536):
            response = self.session.post('/server', json_data={
                'name': TEST_SERVER_NAME + '_test',
                'port': test_port,
            })
            self.assertEqual(response.status_code, 400)

            data = response.json()
            self.assertIn('error', data)
            self.assertEqual(data['error'], 'port_invalid')
            self.assertIn('error_msg', data)


        response = self.session.post('/server', json_data={
            'name': TEST_SERVER_NAME + '_test',
            'dh_param_bits': 512,
        })
        self.assertEqual(response.status_code, 400)

        data = response.json()
        self.assertIn('error', data)
        self.assertEqual(data['error'], 'dh_param_bits_invalid')
        self.assertIn('error_msg', data)


        response = self.session.post('/server', json_data={
            'name': TEST_SERVER_NAME + '_test',
            'mode': 'test',
        })
        self.assertEqual(response.status_code, 400)

        data = response.json()
        self.assertIn('error', data)
        self.assertEqual(data['error'], 'mode_invalid')
        self.assertIn('error_msg', data)


        response = self.session.post('/server', json_data={
            'name': TEST_SERVER_NAME + '_test',
            'dns_servers': ['8.8.8.a'],
        })
        self.assertEqual(response.status_code, 400)

        data = response.json()
        self.assertIn('error', data)
        self.assertEqual(data['error'], 'dns_server_invalid')
        self.assertIn('error_msg', data)


        response = self.session.post('/server', json_data={
            'name': TEST_SERVER_NAME + '_test',
            'protocol': 'a',
        })
        self.assertEqual(response.status_code, 400)

        data = response.json()
        self.assertIn('error', data)
        self.assertEqual(data['error'], 'protocol_invalid')
        self.assertIn('error_msg', data)


class Status(SessionTestCase):
    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_status_get(self):
        response = self.session.get('/status')
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('org_count', data)
        self.assertIn('users_online', data)
        self.assertIn('user_count', data)
        self.assertIn('servers_online', data)
        self.assertIn('server_count', data)
        self.assertIn('server_version', data)
        self.assertIn('public_ip', data)
        self.assertIn('local_networks', data)
        self.assertIn('notification', data)


class User(SessionTestCase):
    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_user_get(self):
        response = self.session.get('/user/%s/%s' % (
            self.org_id, self.user_id))
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.user_id)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_USER_NAME)
        self.assertIn('disabled', data)
        self.assertFalse(data['disabled'])

    def test_user_post_put_get_delete(self):
        response = self.session.post('/user/%s' % self.org_id, json_data={
            'name': TEST_USER_NAME + '2',
            'disabled': True,
        })
        self.assertEqual(response.status_code, 200)
        time.sleep(0.1)

        data = response.json()
        self.assertIn('id', data)
        self.assertRegexpMatches(data['id'], UUID_RE)
        self.assertIn('organization', data)
        self.assertEqual(data['organization'], self.org_id)
        self.assertIn('organization_name', data)
        self.assertEqual(data['organization_name'], TEST_ORG_NAME)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_USER_NAME + '2')
        self.assertIn('disabled', data)
        self.assertTrue(data['disabled'])
        self.assertIn('type', data)
        self.assertIn('otp_secret', data)
        user_id = data['id']


        response = self.session.put('/user/%s/%s' % (self.org_id, user_id),
            json_data={
                'name': TEST_USER_NAME + '3',
            })
        self.assertEqual(response.status_code, 200)
        time.sleep(0.1)

        data = response.json()
        self.assertIn('id', data)
        self.assertRegexpMatches(data['id'], UUID_RE)
        self.assertIn('organization', data)
        self.assertEqual(data['organization'], self.org_id)
        self.assertIn('organization_name', data)
        self.assertEqual(data['organization_name'], TEST_ORG_NAME)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_USER_NAME + '3')
        self.assertIn('disabled', data)
        self.assertTrue(data['disabled'])
        self.assertIn('type', data)
        self.assertIn('otp_secret', data)


        response = self.session.put('/user/%s/%s' % (self.org_id, user_id),
            json_data={
                'disabled': False,
            })
        self.assertEqual(response.status_code, 200)
        time.sleep(0.1)

        data = response.json()
        self.assertIn('id', data)
        self.assertRegexpMatches(data['id'], UUID_RE)
        self.assertIn('organization', data)
        self.assertEqual(data['organization'], self.org_id)
        self.assertIn('organization_name', data)
        self.assertEqual(data['organization_name'], TEST_ORG_NAME)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_USER_NAME + '3')
        self.assertIn('disabled', data)
        self.assertFalse(data['disabled'])
        self.assertIn('type', data)
        self.assertIn('otp_secret', data)


        response = self.session.get('/user/%s' % self.org_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertNotEqual(len(data), 0)
        test_user_found = False
        for user in data:
            self.assertIn('id', user)
            self.assertRegexpMatches(user['id'], UUID_RE)
            self.assertIn('organization', user)
            self.assertIn('organization_name', user)
            self.assertIn('name', user)
            self.assertIn('disabled', user)
            self.assertIn('type', user)
            self.assertIn('status', user)
            self.assertIn('otp_auth', user)
            self.assertIn('otp_secret', user)
            self.assertIn('servers', user)

            if user['name'] == TEST_USER_NAME + '3':
                test_user_found = True
                self.assertFalse(user['disabled'])
                self.assertEqual(user['organization'], self.org_id)
                self.assertEqual(user['organization_name'], TEST_ORG_NAME)
        self.assertTrue(test_user_found)

        response = self.session.delete('/user/%s/%s' % (self.org_id, user_id))
        self.assertEqual(response.status_code, 200)
        time.sleep(0.1)

    @unittest.skipUnless(ENABLE_STANDARD_TESTS, 'Skipping test')
    def test_user_otp_secret_put(self):
        response = self.session.get('/user/%s' % self.org_id)
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertNotEqual(len(data), 0)
        orig_otp_secret = None
        for user in data:
            self.assertIn('id', user)
            if user['id'] == self.user_id:
                orig_otp_secret = user['otp_secret']
        self.assertIsNotNone(orig_otp_secret)

        response = self.session.put('/user/%s/%s/otp_secret' % (
            self.org_id, self.user_id))
        self.assertEqual(response.status_code, 200)
        time.sleep(0.1)

        data = response.json()
        self.assertIn('id', data)
        self.assertEqual(data['id'], self.user_id)
        self.assertIn('organization', data)
        self.assertEqual(data['organization'], self.org_id)
        self.assertIn('organization_name', data)
        self.assertEqual(data['organization_name'], TEST_ORG_NAME)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_USER_NAME)
        self.assertIn('disabled', data)
        self.assertFalse(data['disabled'])
        self.assertIn('type', data)
        self.assertIn('otp_secret', data)
        self.assertNotEqual(data['otp_secret'], orig_otp_secret)
        self.assertRegexpMatches(user['otp_secret'], r'^[A-Z0-9]+$')


class Stress(SessionTestCase):
    def _create_user(self, org_id, name):
        response = self.session.post('/user/%s' % org_id, json_data={
            'name': name,
        })
        self.assertEqual(response.status_code, 200)
        time.sleep(0.1)

        data = response.json()
        self.assertIn('id', data)
        self.assertRegexpMatches(data['id'], UUID_RE)
        self.assertIn('organization', data)
        self.assertEqual(data['organization'], org_id)
        self.assertIn('organization_name', data)
        self.assertEqual(data['organization_name'],
            TEST_ORG_NAME + '_stress')
        self.assertIn('name', data)
        self.assertEqual(data['name'], name)
        self.assertIn('type', data)
        self.assertIn('otp_secret', data)

    @unittest.skipUnless(ENABLE_STRESS_TESTS, 'Skipping stress test')
    def test_user_post_stress(self):
        response = self.session.post('/organization', json_data={
            'name': TEST_ORG_NAME + '_stress',
        })
        self.assertEqual(response.status_code, 200)

        data = response.json()
        self.assertIn('id', data)
        self.assertRegexpMatches(data['id'], UUID_RE)
        self.assertIn('name', data)
        self.assertEqual(data['name'], TEST_ORG_NAME + '_stress')
        org_id = data['id']

        if THREADED_STRESS_TEST:
            num = 0
            for i in xrange(8):
                threads = []

                for x in xrange(512):
                    name = '%s_%s' % (TEST_USER_NAME, str(num).zfill(4))
                    thread = threading.Thread(target=self._create_user,
                        args=(org_id, name))
                    thread.start()
                    threads.append(thread)
                    num += 1

                for thread in threads:
                    thread.join()

        else:
            for i in xrange(4096):
                name = '%s_%s' % (TEST_USER_NAME, str(i).zfill(4))
                self._create_user(org_id, name)


if __name__ == '__main__':
    unittest.main()



from constants import *
from cache import persist_db
from event import Event
from cache_object import CacheObject
import time
import uuid

class LogEntry(CacheObject):
    column_family = 'log_entries'
    str_columns = {'type', 'message'}
    int_columns = {'time'}
    cached_columns = {'type', 'message', 'time'}
    db_instance = persist_db

    def __init__(self, id=None, type=None, message=None):
        CacheObject.__init__(self)

        if id is None:
            self.transaction_start()
            self.id = uuid.uuid4().hex
            self.type = type or INFO
            self.time = int(time.time())
            self.message = message
            self.initialize()
            self.transaction_commit()
        else:
            self.id = id

    def dict(self):
        return {
            'id': self.id,
            'time': self.time,
            'message': self.message,
        }

    def initialize(self):
        while self.db.list_length(self.column_family) >= LOG_LIMIT:
            log_entry_id = self.db.list_index(self.column_family, -1)
            if not log_entry_id:
                break
            LogEntry(id=log_entry_id).remove()
        self.db.list_lpush(self.column_family, self.id)
        Event(type=LOG_UPDATED)

    def remove(self):
        self.transaction_start()
        CacheObject.remove(self)
        self.transaction_commit()

    @classmethod
    def iter_log_entries(cls):
        for log_entry_id in cls.db_instance.list_iter('log_entries'):
            yield cls(id=log_entry_id)



from pritunl.constants import *
from pritunl.log_entry import LogEntry
import pritunl.utils as utils
from pritunl import app_server

@app_server.app.route('/log', methods=['GET'])
@app_server.auth
def log_get():
    log_entries = []

    for log_entry in LogEntry.iter_log_entries():
        log_entries.append(log_entry.dict())

    return utils.jsonify(log_entries)



from pritunl.constants import *
from pritunl.event import Event
import pritunl.utils as utils
from pritunl import app_server
import time
import uuid
import flask

@app_server.app.route('/event', methods=['GET'])
@app_server.app.route('/event/<cursor>', methods=['GET'])
@app_server.auth
def event_get(cursor=None):
    if app_server.interrupt:
        return flask.abort(503)

    events = []
    for event in Event.get_events(cursor):
        events.append(event.dict())

    return utils.jsonify(events)



from constants import *
from exceptions import *
from pritunl import app_server
from cache import cache_db
from werkzeug.http import http_date
import os
import sys
import zlib
import time
import datetime
import mimetypes
import flask

class StaticFile:
    def __init__(self, root, path, cache=True):
        path = '/'.join([x for x in path.split('/') if x and x != '..'])
        path = os.path.normpath(os.path.join(root, path))
        if os.path.commonprefix([root, path]) != root:
            raise InvalidStaticFile('Static path is not a prefix of root path',
                {'path': path})
        if os.path.splitext(path)[1] not in STATIC_FILE_EXTENSIONS:
            raise InvalidStaticFile('Static path file extension is invalid',
                {'path': path})
        self.path = path
        self.cache = cache
        self.data = None
        self.mime_type = None
        self.last_modified = None
        self.etag = None
        self.load_file()

    def get_cache_key(self):
        return 'file_%s' % self.path

    def generate_etag(self, file_name, file_size, mtime):
        file_name = file_name.encode(sys.getfilesystemencoding())
        return 'wzsdm-%d-%s-%s' % (
            time.mktime(mtime.timetuple()),
            file_size,
            zlib.adler32(file_name) & 0xffffffff,
        )

    def set_cache(self):
        cache_db.dict_set(self.get_cache_key(), 'data', self.data or '')
        cache_db.dict_set(self.get_cache_key(), 'mime_type',
            self.mime_type or '')
        cache_db.dict_set(self.get_cache_key(), 'last_modified',
            self.last_modified)
        cache_db.dict_set(self.get_cache_key(), 'etag', self.etag or '')

    def get_cache(self):
        self.data = cache_db.dict_get(self.get_cache_key(), 'data')
        self.mime_type = cache_db.dict_get(self.get_cache_key(), 'mime_type')
        self.last_modified = cache_db.dict_get(self.get_cache_key(),
            'last_modified')
        self.etag = cache_db.dict_get(self.get_cache_key(), 'etag')

    def load_file(self):
        if app_server.static_cache and cache_db.exists(self.get_cache_key()):
            self.get_cache()
            return

        if not os.path.isfile(self.path):
            if app_server.static_cache:
                self.set_cache()
            return

        file_basename = os.path.basename(self.path)
        file_mtime = datetime.datetime.utcfromtimestamp(
            os.path.getmtime(self.path))
        file_size = int(os.path.getsize(self.path))

        with open(self.path, 'r') as static_file:
            self.data = static_file.read()

        self.mime_type = mimetypes.guess_type(file_basename)[0] or 'text/plain'
        self.last_modified = http_date(file_mtime)
        self.etag = self.generate_etag(file_basename, file_size, file_mtime)
        if app_server.static_cache:
            self.set_cache()

    def get_response(self):
        if not self.last_modified:
            flask.abort(404)
        response = flask.Response(response=self.data, mimetype=self.mime_type)
        if app_server.static_cache and not app_server.debug and self.cache:
            response.headers.add('Cache-Control',
                'max-age=%s, public' % STATIC_CACHE_TIME)
            response.headers.add('ETag', '"%s"' % self.etag)
        else:
            response.headers.add('Cache-Control',
                'no-cache, no-store, must-revalidate')
            response.headers.add('Pragma', 'no-cache')
            response.headers.add('Expires', 0)
        response.headers.add('Last-Modified', self.last_modified)
        return response



from pritunl.constants import *
from pritunl.exceptions import *
from pritunl.organization import Organization
from pritunl.event import Event
from pritunl.log_entry import LogEntry
import pritunl.utils as utils
from pritunl import app_server
import flask
import math
import time

@app_server.app.route('/user/<org_id>', methods=['GET'])
@app_server.app.route('/user/<org_id>/<user_id>', methods=['GET'])
@app_server.app.route('/user/<org_id>/<int:page>', methods=['GET'])
@app_server.auth
def user_get(org_id, user_id=None, page=None):
    org = Organization.get_org(id=org_id)
    if user_id:
        return utils.jsonify(org.get_user(user_id).dict())
    else:
        page = flask.request.args.get('page', None)
        page = int(page) if page else page
        search = flask.request.args.get('search', None)
        limit = int(flask.request.args.get('limit', USER_PAGE_COUNT))
        otp_auth = False
        search_more = True
        server_count = 0
        clients = {}
        servers = []

        for server in org.iter_servers():
            servers.append(server)
            server_count += 1
            if server.otp_auth:
                otp_auth = True
            server_clients = server.clients
            for client_id in server_clients:
                client = server_clients[client_id]
                if client_id not in clients:
                    clients[client_id] = {}
                clients[client_id][server.id] = client

        users = []
        for user in org.iter_users(page=page, prefix=search,
                prefix_limit=limit):
            if user is None:
                search_more = False
                break
            is_client = user.id in clients
            user_dict = user.dict()
            user_dict['status'] = is_client
            user_dict['otp_auth'] = otp_auth
            server_data = []
            for server in servers:
                local_ip_addr, remote_ip_addr = server.get_ip_set(
                    org_id, user.id)
                data = {
                    'id': server.id,
                    'name': server.name,
                    'status': is_client and server.id in clients[user.id],
                    'local_address': local_ip_addr,
                    'remote_address': remote_ip_addr,
                    'real_address': None,
                    'virt_address': None,
                    'bytes_received': None,
                    'bytes_sent': None,
                    'connected_since': None,
                }
                if is_client:
                    client_data = clients[user.id].get(server.id)
                    if client_data:
                        data.update(client_data)
                server_data.append(data)
            user_dict['servers'] = server_data
            users.append(user_dict)

        if page is not None:
            return utils.jsonify({
                'page': page,
                'page_total': org.page_total,
                'server_count': server_count,
                'users': users,
            })
        elif search is not None:
            return utils.jsonify({
                'search': search,
                'search_more': search_more,
                'search_limit': limit,
                'search_count': org.get_last_prefix_count(),
                'search_time':  round((time.time() - flask.g.start), 4),
                'server_count': server_count,
                'users': users,
            })
        else:
            return utils.jsonify(users)

@app_server.app.route('/user/<org_id>', methods=['POST'])
@app_server.auth
def user_post(org_id):
    org = Organization.get_org(id=org_id)

    if isinstance(flask.request.json, list):
        users = []
        for user_data in flask.request.json:
            name = utils.filter_str(user_data['name'])
            email = utils.filter_str(user_data.get('email'))
            user = org.new_user(type=CERT_CLIENT, name=name, email=email)

            disabled = user_data.get('disabled')
            if disabled is not None:
                user.disabled = disabled
                user.commit()

            users.append(user.dict())
        return utils.jsonify(users)
    else:
        name = utils.filter_str(flask.request.json['name'])
        email = None
        if 'email' in flask.request.json:
            email = utils.filter_str(flask.request.json['email'])
        user = org.new_user(type=CERT_CLIENT, name=name, email=email)

        disabled = flask.request.json.get('disabled')
        if disabled is not None:
            user.disabled = disabled
            user.commit()

        return utils.jsonify(user.dict())

@app_server.app.route('/user/<org_id>/<user_id>', methods=['PUT'])
@app_server.auth
def user_put(org_id, user_id):
    org = Organization.get_org(id=org_id)
    user = org.get_user(user_id)

    name = flask.request.json.get('name')
    if name:
        name = utils.filter_str(name)

    if 'email' in flask.request.json:
        email = flask.request.json['email']
        if email:
            user.email = utils.filter_str(email)
        else:
            user.email = None

    disabled = flask.request.json.get('disabled')
    if disabled is not None:
        user.disabled = disabled

    if name:
        user.rename(name)
    else:
        user.commit()
        Event(type=USERS_UPDATED, resource_id=user.org.id)

    if disabled:
        if user.type == CERT_CLIENT:
            LogEntry(message='Disabled user "%s".' % user.name)

        for server in org.iter_servers():
            server_clients = server.clients
            if user_id in server_clients:
                server.restart()
    elif disabled == False and user.type == CERT_CLIENT:
        LogEntry(message='Enabled user "%s".' % user.name)

    send_key_email = flask.request.json.get('send_key_email')
    if send_key_email and user.email:
        try:
            user.send_key_email(send_key_email)
        except EmailNotConfiguredError:
            return utils.jsonify({
                'error': EMAIL_NOT_CONFIGURED,
                'error_msg': EMAIL_NOT_CONFIGURED_MSG,
            }, 400)
        except EmailFromInvalid:
            return utils.jsonify({
                'error': EMAIL_FROM_INVALID,
                'error_msg': EMAIL_FROM_INVALID_MSG,
            }, 400)
        except EmailApiKeyInvalid:
            return utils.jsonify({
                'error': EMAIL_API_KEY_INVALID,
                'error_msg': EMAIL_API_KEY_INVALID_MSG,
            }, 400)

    return utils.jsonify(user.dict())

@app_server.app.route('/user/<org_id>/<user_id>', methods=['DELETE'])
@app_server.auth
def user_delete(org_id, user_id):
    org = Organization.get_org(id=org_id)
    user = org.get_user(user_id)
    user_id = user.id
    user.remove()

    for server in org.iter_servers():
        server_clients = server.clients
        if user_id in server_clients:
            server.restart()

    return utils.jsonify({})

@app_server.app.route('/user/<org_id>/<user_id>/otp_secret', methods=['PUT'])
@app_server.auth
def user_otp_secret_put(org_id, user_id):
    org = Organization.get_org(id=org_id)
    user = org.get_user(user_id)
    user.generate_otp_secret()
    return utils.jsonify(user.dict())



class BaseError(Exception):
    def __init__(self, message, data):
        self.__dict__.update(data)
        message = '%s. %r' % (message, data)
        Exception.__init__(self, message)


class UserError(BaseError):
    pass

class KeyLinkError(UserError):
    pass


class EmailError(BaseError):
    pass

class EmailNotConfiguredError(BaseError):
    pass

class EmailFromInvalid(EmailError):
    pass

class EmailApiKeyInvalid(EmailError):
    pass


class ServerError(BaseError):
    pass

class ServerMissingOrg(ServerError):
    pass

class ServerStartError(ServerError):
    pass

class ServerStopError(ServerError):
    pass

class InvalidNodeAPIKey(ServerError):
    pass

class NodeConnectionError(ServerError):
    pass

class IptablesError(ServerError):
    pass

class InvalidStaticFile(ServerError):
    pass



from constants import *
from cache import cache_db
import collections
import re

_keys = collections.defaultdict(
    lambda: collections.defaultdict(lambda: collections.Counter()))
_values = collections.defaultdict(
    lambda: collections.defaultdict(lambda: set()))

class CacheTrie(object):
    __slots__ = ('name', 'key')

    def __init__(self, name, key=''):
        self.name = name
        self.key = key

    def clear_cache(self):
        _keys.pop(self.name, None)
        _values.pop(self.name, None)

    def add_key(self, key, value):
        keys = _keys[self.name]
        cur_key = self.key
        new_key = cur_key
        for char in key.lower():
            new_key += char
            keys[cur_key][new_key] += 1
            cur_key = new_key
        _values[self.name][cur_key].add(value)

    def add_key_terms(self, key, value):
        for term in re.split('[^a-z0-9]', key.lower()):
            self.add_key(term, value)
        self.add_key(key, value)

    def remove_key(self, key, value):
        keys = _keys[self.name]
        values = _values[self.name]
        cur_key = self.key
        new_key = cur_key
        for char in key.lower():
            new_key += char
            keys[cur_key][new_key] -= 1
            if not keys[cur_key][new_key]:
                keys[cur_key].pop(new_key, None)
                if not keys[cur_key]:
                    keys.pop(cur_key, None)
            cur_key = new_key
        try:
            values[cur_key].remove(value)
        except KeyError:
            pass
        if not values[cur_key]:
            values.pop(cur_key, None)

    def remove_key_terms(self, key, value):
        for term in re.split('[^a-z0-9]', key.lower()):
            self.remove_key(term, value)
        self.remove_key(key, value)

    def chain(self, node_values):
        name = self.name
        key = self.key
        keys = _keys[name]
        values = _values[name]
        if key in keys:
            for node_key in keys[key]:
                CacheTrie(name, node_key).chain(node_values)
        if key in values:
            node_values.update(values[key])
        return node_values

    def get_prefix(self, prefix):
        return CacheTrie(self.name, prefix.lower()).chain(set())

    def iter_prefix(self, prefix):
        for value in CacheTrie(self.name, prefix.lower()).chain(set()):
            yield value



from pritunl.constants import *
import pritunl.utils as utils
from pritunl import app_server
from pritunl.cache import persist_db
import flask
import re

@app_server.app.route('/subscription', methods=['GET'])
@app_server.auth
def subscription_get():
    app_server.subscription_update()
    return utils.jsonify(app_server.subscription_dict())

@app_server.app.route('/subscription/state', methods=['GET'])
def subscription_state_get():
    app_server.subscription_update()
    return utils.jsonify({
        'disabled': SUB_OFF,
        'active': app_server.sub_active,
    })

@app_server.app.route('/subscription', methods=['POST'])
@app_server.auth
def subscription_post():
    license = flask.request.json['license']
    license = license.lower().replace('begin license', '').replace(
        'end license', '')
    license = re.sub(r'[\W_]+', '', license)

    try:
        response = utils.request.get(SUBSCRIPTION_SERVER,
            json_data={'license': license})
    except httplib.HTTPException:
        return utils.jsonify({
            'error': SUBSCRIPTION_SERVER_ERROR,
            'error_msg': SUBSCRIPTION_SERVER_ERROR_MSG,
        }, 500)
    data = response.json()

    if response.status_code != 200:
        return utils.jsonify(data, response.status_code);

    persist_db.set('license', license)
    app_server.subscription_update()
    return utils.jsonify(app_server.subscription_dict())

@app_server.app.route('/subscription', methods=['PUT'])
@app_server.auth
def subscription_put():
    card = flask.request.json.get('card')
    email = flask.request.json.get('email')
    cancel = flask.request.json.get('cancel')

    try:
        if cancel:
            response = utils.request.delete(SUBSCRIPTION_SERVER,
                json_data={
                    'license': persist_db.get('license'),
                },
            )
        else:
            response = utils.request.put(SUBSCRIPTION_SERVER,
                json_data={
                    'license': persist_db.get('license'),
                    'card': card,
                    'email': email,
                },
            )
    except httplib.HTTPException:
        return utils.jsonify({
            'error': SUBSCRIPTION_SERVER_ERROR,
            'error_msg': SUBSCRIPTION_SERVER_ERROR_MSG,
        }, 500)

    if response.status_code != 200:
        return utils.jsonify(response.json(), response.status_code)

    app_server.subscription_update()
    return utils.jsonify(app_server.subscription_dict())

@app_server.app.route('/subscription', methods=['DELETE'])
@app_server.auth
def subscription_delete():
    persist_db.remove('license')
    app_server.subscription_update()
    return utils.jsonify({})



from pritunl.constants import *
from pritunl.organization import Organization
import pritunl.utils as utils
from pritunl import app_server
import flask

@app_server.app.route('/organization', methods=['GET'])
@app_server.app.route('/organization/<org_id>', methods=['GET'])
@app_server.auth
def org_get(org_id=None):
    if org_id:
        return utils.jsonify(Organization.get_org(id=org_id).dict())
    else:
        orgs = []
        for org in Organization.iter_orgs():
            orgs.append(org.dict())
        return utils.jsonify(orgs)

@app_server.app.route('/organization', methods=['POST'])
@app_server.auth
def org_post():
    name = utils.filter_str(flask.request.json['name'])
    org = Organization(name=name)
    return utils.jsonify(org.dict())

@app_server.app.route('/organization/<org_id>', methods=['PUT'])
@app_server.auth
def org_put(org_id):
    org = Organization.get_org(id=org_id)
    name = utils.filter_str(flask.request.json['name'])
    org.rename(name)
    return utils.jsonify(org.dict())

@app_server.app.route('/organization/<org_id>', methods=['DELETE'])
@app_server.auth
def org_delete(org_id):
    org = Organization.get_org(id=org_id)
    org.remove()
    return utils.jsonify({})



from pritunl.constants import *
from pritunl.exceptions import *
from pritunl.server import Server
from pritunl.node_server import NodeServer
from pritunl.organization import Organization
from pritunl.log_entry import LogEntry
from event import Event
import pritunl.utils as utils
from pritunl import app_server
import flask
import re
import random

def _network_invalid():
    return utils.jsonify({
        'error': NETWORK_INVALID,
        'error_msg': NETWORK_INVALID_MSG,
    }, 400)

def _interface_invalid():
        return utils.jsonify({
            'error': INTERFACE_INVALID,
            'error_msg': INTERFACE_INVALID_MSG,
        }, 400)

def _port_invalid():
        return utils.jsonify({
            'error': PORT_INVALID,
            'error_msg': PORT_INVALID_MSG,
        }, 400)

def _dh_param_bits_invalid():
        return utils.jsonify({
            'error': DH_PARAM_BITS_INVALID,
            'error_msg': DH_PARAM_BITS_INVALID_MSG,
        }, 400)

def _local_network_invalid():
    return utils.jsonify({
        'error': LOCAL_NETWORK_INVALID,
        'error_msg': LOCAL_NETWORK_INVALID_MSG,
    }, 400)

def _dns_server_invalid():
    return utils.jsonify({
        'error': DNS_SERVER_INVALID,
        'error_msg': DNS_SERVER_INVALID_MSG,
    }, 400)

@app_server.app.route('/server', methods=['GET'])
@app_server.app.route('/server/<server_id>', methods=['GET'])
@app_server.auth
def server_get(server_id=None):
    if server_id:
        return utils.jsonify(Server.get_server(server_id).dict())
    else:
        servers = []
        servers_dict = {}
        servers_sort = []

        for server in Server.iter_servers():
            servers.append(server.dict())

        return utils.jsonify(servers)

@app_server.app.route('/server', methods=['POST'])
@app_server.app.route('/server/<server_id>', methods=['PUT'])
@app_server.auth
def server_put_post(server_id=None):
    network_used = set()
    interface_used = set()
    port_used = set()
    for server in Server.iter_servers():
        if server.id == server_id:
            continue
        network_used.add(server.network)
        interface_used.add(server.interface)
        port_used.add('%s%s' % (server.port, server.protocol))

    name = None
    name_def = False
    if 'name' in flask.request.json:
        name_def = True
        name = utils.filter_str(flask.request.json['name'])

    type = SERVER
    type_def = False
    if 'type' in flask.request.json:
        type_def = True
        if flask.request.json['type'] == NODE_SERVER:
            type = NODE_SERVER

    network = None
    network_def = False
    if 'network' in flask.request.json:
        network_def = True
        network = flask.request.json['network']

        if network not in SAFE_PUB_SUBNETS:
            network_split = network.split('/')
            if len(network_split) != 2:
                return _network_invalid()

            address = network_split[0].split('.')
            if len(address) != 4:
                return _network_invalid()
            for i, value in enumerate(address):
                try:
                    address[i] = int(value)
                except ValueError:
                    return _network_invalid()
            if address[0] != 10:
                return _network_invalid()

            if address[1] > 255 or address[1] < 0 or \
                    address[2] > 255 or address[2] < 0:
                return _network_invalid()

            if address[3] != 0:
                return _network_invalid()

            try:
                subnet = int(network_split[1])
            except ValueError:
                return _network_invalid()

            if subnet < 8 or subnet > 24:
                return _network_invalid()

    interface = None
    interface_def = False
    if 'interface' in flask.request.json:
        interface_def = True
        interface = flask.request.json['interface']

        if not re.match('^[a-z0-9]+$', interface):
            return _interface_invalid()

        if interface[:3] != 'tun':
            return _interface_invalid()

        try:
            interface_num = int(interface[3:])
        except ValueError:
            return _interface_invalid()

        if interface_num > 64:
            return _interface_invalid()

        interface = interface[:3] + str(interface_num)

    protocol = 'udp'
    protocol_def = False
    if 'protocol' in flask.request.json:
        protocol_def = True
        protocol = flask.request.json['protocol'].lower()

        if protocol not in ('udp', 'tcp'):
            return utils.jsonify({
                'error': PROTOCOL_INVALID,
                'error_msg': PROTOCOL_INVALID_MSG,
            }, 400)

    port = None
    port_def = False
    if 'port' in flask.request.json:
        port_def = True
        port = flask.request.json['port']

        try:
            port = int(port)
        except ValueError:
            return _port_invalid()

        if port < 1 or port > 65535:
            return _port_invalid()

    dh_param_bits = None
    dh_param_bits_def = False
    if 'dh_param_bits' in flask.request.json:
        dh_param_bits_def = True
        dh_param_bits = flask.request.json['dh_param_bits']

        try:
            dh_param_bits = int(dh_param_bits)
        except ValueError:
            return _dh_param_bits_invalid()

        if dh_param_bits not in VALID_DH_PARAM_BITS:
            return _dh_param_bits_invalid()

    mode = None
    mode_def = False
    if 'mode' in flask.request.json:
        mode_def = True
        mode = flask.request.json['mode']

        if mode not in (ALL_TRAFFIC, LOCAL_TRAFFIC, VPN_TRAFFIC):
            return utils.jsonify({
                'error': MODE_INVALID,
                'error_msg': MODE_INVALID_MSG,
            }, 400)

    local_networks = None
    local_networks_def = False
    if 'local_networks' in flask.request.json:
        local_networks_def = True
        local_networks = flask.request.json['local_networks'] or []

        for local_network in local_networks:
            local_network_split = local_network.split('/')
            if len(local_network_split) != 2:
                return _local_network_invalid()

            address = local_network_split[0].split('.')
            if len(address) != 4:
                return _local_network_invalid()
            for i, value in enumerate(address):
                try:
                    address[i] = int(value)
                except ValueError:
                    return _local_network_invalid()
            if address[0] > 255 or address[0] < 0 or \
                    address[1] > 255 or address[1] < 0 or \
                    address[2] > 255 or address[2] < 0 or \
                    address[3] > 254 or address[3] < 0:
                return _local_network_invalid()

            try:
                subnet = int(local_network_split[1])
            except ValueError:
                return _local_network_invalid()

            if subnet < 8 or subnet > 30:
                return _local_network_invalid()

    dns_servers = None
    dns_servers_def = False
    if 'dns_servers' in flask.request.json:
        dns_servers_def = True
        dns_servers = flask.request.json['dns_servers'] or []

        for dns_server in dns_servers:
            if not re.match(IP_REGEX, dns_server):
                return _dns_server_invalid()

    search_domain = None
    search_domain_def = False
    if 'search_domain' in flask.request.json:
        search_domain_def = True
        search_domain = utils.filter_str(flask.request.json['search_domain'])

    public_address = None
    public_address_def = False
    if 'public_address' in flask.request.json:
        public_address_def = True
        public_address = utils.filter_str(flask.request.json['public_address'])

    debug = False
    debug_def = False
    if 'debug' in flask.request.json:
        debug_def = True
        debug = True if flask.request.json['debug'] else False

    otp_auth = False
    otp_auth_def = False
    if 'otp_auth' in flask.request.json:
        otp_auth_def = True
        otp_auth = True if flask.request.json['otp_auth'] else False

    lzo_compression = False
    lzo_compression_def = False
    if 'lzo_compression' in flask.request.json:
        lzo_compression_def = True
        lzo_compression = True if flask.request.json[
            'lzo_compression'] else False

    node_host = None
    node_host_def = False
    if 'node_host' in flask.request.json:
        node_host_def = True
        node_host = utils.filter_str(flask.request.json['node_host'])

    node_port = None
    node_port_def = False
    if 'node_port' in flask.request.json:
        node_port_def = True
        node_port = flask.request.json['node_port']

        if node_port is not None:
            try:
                node_port = int(node_port)
            except ValueError:
                return _port_invalid()

            if node_port < 1 or node_port > 65535:
                return _port_invalid()

    node_key = None
    node_key_def = False
    if 'node_key' in flask.request.json:
        node_key_def = True
        node_key = utils.filter_str(flask.request.json['node_key'])

    if not server_id:
        if not name_def:
            return utils.jsonify({
                'error': MISSING_PARAMS,
                'error_msg': MISSING_PARAMS_MSG,
            }, 400)

        if not network_def:
            network_def = True
            for i in xrange(5000):
                rand_network = '10.%s.%s.0/24' % (
                    random.randint(15,250), random.randint(15,250))
                if rand_network not in network_used:
                    network = rand_network
                    break
            if not network:
                return utils.jsonify({
                    'error': NETWORK_IN_USE,
                    'error_msg': NETWORK_IN_USE_MSG,
                }, 400)

        if not interface_def:
            interface_def = True
            for i in xrange(64):
                rand_interface = 'tun%s' % i
                if rand_interface not in interface_used:
                    interface = rand_interface
                    break
            if not interface:
                return utils.jsonify({
                    'error': INTERFACE_IN_USE,
                    'error_msg': INTERFACE_IN_USE_MSG,
                }, 400)

        if not port_def:
            port_def = True
            rand_ports = range(10000, 19999)
            random.shuffle(rand_ports)
            for rand_port in rand_ports:
                if '%s%s' % (rand_port, protocol) not in port_used:
                    port = rand_port
                    break
            if not port:
                return utils.jsonify({
                    'error': PORT_PROTOCOL_IN_USE,
                    'error_msg': PORT_PROTOCOL_IN_USE_MSG,
                }, 400)

        if not dh_param_bits_def:
            dh_param_bits_def = True
            dh_param_bits = DEFAULT_DH_PARAM_BITS

        if not mode_def:
            mode_def = True
            if local_networks_def and local_networks:
                mode = LOCAL_TRAFFIC
            else:
                mode = ALL_TRAFFIC

        if not public_address_def:
            public_address_def = True
            public_address = app_server.public_ip

    if network_def:
        if network in network_used:
            return utils.jsonify({
                'error': NETWORK_IN_USE,
                'error_msg': NETWORK_IN_USE_MSG,
            }, 400)
    if interface_def:
        if interface in interface_used:
            return utils.jsonify({
                'error': INTERFACE_IN_USE,
                'error_msg': INTERFACE_IN_USE_MSG,
            }, 400)
    if port_def:
        if '%s%s' % (port, protocol) in port_used:
            return utils.jsonify({
                'error': PORT_PROTOCOL_IN_USE,
                'error_msg': PORT_PROTOCOL_IN_USE_MSG,
            }, 400)

    if not server_id:
        if type == NODE_SERVER:
            server = NodeServer(
                name=name,
                network=network,
                interface=interface,
                port=port,
                protocol=protocol,
                dh_param_bits=dh_param_bits,
                mode=mode,
                local_networks=local_networks,
                dns_servers=dns_servers,
                search_domain=search_domain,
                public_address=public_address,
                otp_auth=otp_auth,
                lzo_compression=lzo_compression,
                debug=debug,
                node_host=node_host,
                node_port=node_port,
                node_key=node_key,
            )
        else:
            server = Server(
                name=name,
                network=network,
                interface=interface,
                port=port,
                protocol=protocol,
                dh_param_bits=dh_param_bits,
                mode=mode,
                local_networks=local_networks,
                dns_servers=dns_servers,
                search_domain=search_domain,
                public_address=public_address,
                otp_auth=otp_auth,
                lzo_compression=lzo_compression,
                debug=debug,
            )
    else:
        server = Server.get_server(id=server_id)
        if server.status:
            return utils.jsonify({
                'error': SERVER_NOT_OFFLINE,
                'error_msg': SERVER_NOT_OFFLINE_SETTINGS_MSG,
            }, 400)
        if name_def:
            server.name = name
        if network_def:
            server.network = network
        if interface_def:
            server.interface = interface
        if port_def:
            server.port = port
        if protocol_def:
            server.protocol = protocol
        if dh_param_bits_def:
            server.dh_param_bits = dh_param_bits
        if mode_def:
            server.mode = mode
        if local_networks_def:
            server.local_networks = local_networks
        if dns_servers_def:
            server.dns_servers = dns_servers
        if search_domain_def:
            server.search_domain = search_domain
        if public_address_def:
            server.public_address = public_address
        if otp_auth_def:
            server.otp_auth = otp_auth
        if lzo_compression_def:
            server.lzo_compression = lzo_compression
        if debug_def:
            server.debug = debug
        if server.type == NODE_SERVER:
            if node_host_def:
                server.node_host = node_host
            if node_port_def:
                server.node_port = node_port
            if node_key_def:
                server.node_key = node_key
        server.commit()

    for org in server.iter_orgs():
        Event(type=USERS_UPDATED, resource_id=org.id)
    return utils.jsonify(server.dict())

@app_server.app.route('/server/<server_id>', methods=['DELETE'])
@app_server.auth
def server_delete(server_id):
    server = Server.get_server(id=server_id)
    server.remove()
    return utils.jsonify({})

@app_server.app.route('/server/<server_id>/organization', methods=['GET'])
@app_server.auth
def server_org_get(server_id):
    orgs = []
    server = Server.get_server(id=server_id)
    for org in server.iter_orgs():
        orgs.append({
            'id': org.id,
            'server': server.id,
            'name': org.name,
        })
    return utils.jsonify(orgs)

@app_server.app.route('/server/<server_id>/organization/<org_id>',
    methods=['PUT'])
@app_server.auth
def server_org_put(server_id, org_id):
    server = Server.get_server(id=server_id)
    if server.status:
        return utils.jsonify({
            'error': SERVER_NOT_OFFLINE,
            'error_msg': SERVER_NOT_OFFLINE_ATTACH_ORG_MSG,
        }, 400)
    org = server.add_org(org_id)
    return utils.jsonify({
        'id': org.id,
        'server': server.id,
        'name': org.name,
    })

@app_server.app.route('/server/<server_id>/organization/<org_id>',
    methods=['DELETE'])
@app_server.auth
def server_org_delete(server_id, org_id):
    server = Server.get_server(id=server_id)
    if server.status:
        return utils.jsonify({
            'error': SERVER_NOT_OFFLINE,
            'error_msg': SERVER_NOT_OFFLINE_DETACH_ORG_MSG,
        }, 400)
    server.remove_org(org_id)
    return utils.jsonify({})

@app_server.app.route('/server/<server_id>/<operation>', methods=['PUT'])
@app_server.auth
def server_operation_put(server_id, operation):
    server = Server.get_server(id=server_id)

    try:
        if operation == START:
            server.start()
        if operation == STOP:
            server.stop()
        elif operation == RESTART:
            server.restart()
    except NodeConnectionError:
        return utils.jsonify({
            'error': NODE_CONNECTION_ERROR,
            'error_msg': NODE_CONNECTION_ERROR_MSG,
        }, 400)
    except InvalidNodeAPIKey:
        return utils.jsonify({
            'error': NODE_API_KEY_INVLID,
            'error_msg': NODE_API_KEY_INVLID_MSG,
        }, 400)

    return utils.jsonify(server.dict())

@app_server.app.route('/server/<server_id>/output', methods=['GET'])
@app_server.auth
def server_output_get(server_id):
    server = Server.get_server(id=server_id)
    return utils.jsonify({
        'id': server.id,
        'output': server.get_output(),
    })

@app_server.app.route('/server/<server_id>/output', methods=['DELETE'])
@app_server.auth
def server_output_delete(server_id):
    server = Server.get_server(id=server_id)
    server.clear_output()
    return utils.jsonify({})

@app_server.app.route('/server/<server_id>/bandwidth', methods=['GET'])
@app_server.app.route('/server/<server_id>/bandwidth/<period>',
    methods=['GET'])
@app_server.auth
def server_bandwidth_get(server_id, period=None):
    server = Server.get_server(id=server_id)
    return utils.jsonify(server.get_bandwidth(period=period))

@app_server.app.route('/server/<server_id>/tls_verify', methods=['POST'])
@app_server.local_auth
def server_tls_verify_post(server_id):
    org_id = flask.request.json['org_id']
    user_id = flask.request.json['user_id']

    server = Server(server_id)
    if not server:
        return utils.jsonify({
            'error': SERVER_INVALID,
            'error_msg': SERVER_INVALID_MSG,
        }, 401)
    org = server.get_org(org_id)
    if not org:
        LogEntry(message='User failed authentication, ' +
            'invalid organization "%s".' % server.name)
        return utils.jsonify({
            'error': ORG_INVALID,
            'error_msg': ORG_INVALID_MSG,
        }, 401)
    user = org.get_user(user_id)
    if not user:
        LogEntry(message='User failed authentication, ' +
            'invalid user "%s".' % server.name)
        return utils.jsonify({
            'error': USER_INVALID,
            'error_msg': USER_INVALID_MSG,
        }, 401)
    if user.disabled:
        LogEntry(message='User failed authentication, ' +
            'disabled user "%s".' % server.name)
        return utils.jsonify({
            'error': USER_INVALID,
            'error_msg': USER_INVALID_MSG,
        }, 401)

    return utils.jsonify({
        'authenticated': True,
    })

@app_server.app.route('/server/<server_id>/otp_verify', methods=['POST'])
@app_server.local_auth
def server_otp_verify_post(server_id):
    org_id = flask.request.json['org_id']
    user_id = flask.request.json['user_id']
    otp_code = flask.request.json['otp_code']
    remote_ip = flask.request.json.get('remote_ip')

    server = Server(server_id)
    if not server:
        return utils.jsonify({
            'error': SERVER_INVALID,
            'error_msg': SERVER_INVALID_MSG,
        }, 401)
    org = server.get_org(org_id)
    if not org:
        LogEntry(message='User failed authentication, ' +
            'invalid organization "%s".' % server.name)
        return utils.jsonify({
            'error': ORG_INVALID,
            'error_msg': ORG_INVALID_MSG,
        }, 401)
    user = org.get_user(user_id)
    if not user:
        LogEntry(message='User failed authentication, ' +
            'invalid user "%s".' % server.name)
        return utils.jsonify({
            'error': USER_INVALID,
            'error_msg': USER_INVALID_MSG,
        }, 401)
    if not user.verify_otp_code(otp_code, remote_ip):
        LogEntry(message='User failed two-step authentication "%s".' % (
            user.name))
        return utils.jsonify({
            'error': OTP_CODE_INVALID,
            'error_msg': OTP_CODE_INVALID_MSG,
        }, 401)

    return utils.jsonify({
        'authenticated': True,
    })

@app_server.app.route('/server/<server_id>/client_connect', methods=['POST'])
@app_server.local_auth
def server_client_connect_post(server_id):
    org_id = flask.request.json['org_id']
    user_id = flask.request.json['user_id']

    server = Server(server_id)
    if not server:
        return utils.jsonify({
            'error': SERVER_INVALID,
            'error_msg': SERVER_INVALID_MSG,
        }, 401)
    org = server.get_org(org_id)
    if not org:
        return utils.jsonify({
            'error': ORG_INVALID,
            'error_msg': ORG_INVALID_MSG,
        }, 401)
    user = org.get_user(user_id)
    if not user:
        return utils.jsonify({
            'error': USER_INVALID,
            'error_msg': USER_INVALID_MSG,
        }, 401)
    if user.type != CERT_CLIENT:
        return utils.jsonify({
            'error': USER_TYPE_INVALID,
            'error_msg': USER_TYPE_INVALID_MSG,
        }, 401)

    local_ip_addr, remote_ip_addr = server.get_ip_set(org.id, user_id)
    if local_ip_addr and remote_ip_addr:
        client_conf = 'ifconfig-push %s %s' % (local_ip_addr, remote_ip_addr)
    else:
        client_conf = ''

    return utils.jsonify({
        'client_conf': client_conf,
    })

@app_server.app.route('/server/<server_id>/client_disconnect',
    methods=['POST'])
@app_server.local_auth
def server_client_disconnect_post(server_id):
    org_id = flask.request.json['org_id']
    user_id = flask.request.json['user_id']

    server = Server(server_id)
    if not server:
        return utils.jsonify({
            'error': SERVER_INVALID,
            'error_msg': SERVER_INVALID_MSG,
        }, 401)
    org = server.get_org(org_id)
    if not org:
        return utils.jsonify({
            'error': ORG_INVALID,
            'error_msg': ORG_INVALID_MSG,
        }, 401)
    user = org.get_user(user_id)
    if not user:
        return utils.jsonify({
            'error': USER_INVALID,
            'error_msg': USER_INVALID_MSG,
        }, 401)

    return utils.jsonify({})



from constants import *
from exceptions import *
from pritunl import app_server
from cache import cache_db, persist_db
from cache_trie import CacheTrie
from config import Config
from log_entry import LogEntry
from event import Event
import uuid
import tarfile
import os
import subprocess
import logging
import hashlib
import base64
import utils
import struct
import hmac
import time
import threading
import json

logger = logging.getLogger(APP_NAME)

class User(Config):
    str_options = {'name', 'email', 'otp_secret', 'type'}
    bool_options = {'disabled'}
    default_options = {
        'name': 'undefined',
        'disabled': False,
    }
    chmod_mode = 0600
    cached = True
    cache_prefix = 'user'

    def __init__(self, org, id=None, name=None, email=None, type=None):
        Config.__init__(self)
        self.org = org

        if id is None:
            if type == CERT_CA:
                self.id = CA_CERT_ID
            elif type is None:
                raise AttributeError('Type must be specified')
            else:
                self.id = uuid.uuid4().hex
            self.name = name
            self.email = email
            self.type = type
        else:
            self.id = id

        if id is None:
            self._initialize()

    def __setattr__(self, name, value):
        if name == 'id':
            self.reqs_path = os.path.join(self.org.path, REQS_DIR,
                '%s.csr' % value)
            self.key_path = os.path.join(self.org.path, KEYS_DIR,
                '%s.key' % value)
            self.cert_path = os.path.join(self.org.path, CERTS_DIR,
                '%s.crt' % value)
            self.temp_key_archive_path = os.path.join(self.org.path,
                TEMP_DIR, '%s_%s.tar' % (value, uuid.uuid4().hex))
            self.set_path(os.path.join(self.org.path, USERS_DIR,
                '%s.conf' % value))

            self.temp_path = os.path.join(self.org.path, TEMP_DIR, value)
            self.ssl_conf_path = os.path.join(self.temp_path, OPENSSL_NAME)
            self.index_path = os.path.join(self.temp_path, INDEX_NAME)
            self.index_attr_path = os.path.join(
                self.temp_path, INDEX_ATTR_NAME)
            self.serial_path = os.path.join(self.temp_path, SERIAL_NAME)
        Config.__setattr__(self, name, value)

    def dict(self):
        return {
            'id': self.id,
            'organization': self.org.id,
            'organization_name': self.org.name,
            'name': self.name,
            'email': self.email,
            'type': self.type,
            'otp_secret': self.otp_secret,
            'disabled': self.disabled,
        }

    def _upgrade_0_10_9(self):
        if not self.type:
            logger.debug('Upgrading user to v0.10.9... %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
            })
            with open(self.cert_path, 'r') as cert_file:
                cert_data = cert_file.read()
                if 'CA:TRUE' in cert_data:
                    self.type = CERT_CA
                elif 'TLS Web Server Authentication' in cert_data:
                    self.type = CERT_SERVER
                else:
                    self.type = CERT_CLIENT
            self.commit()

    def _initialize(self):
        pool_user_id = None
        if self.type == CERT_CLIENT:
            pool_user_id = cache_db.set_pop(
                self.org.get_cache_key('users_client_pool'))
        elif self.type == CERT_SERVER:
            pool_user_id = cache_db.set_pop(
                self.org.get_cache_key('users_server_pool'))

        if pool_user_id:
            cache_db.publish('pooler', 'update')
            self.id = pool_user_id
            self.commit()
        else:
            self._setup_openssl()
            task_id = uuid.uuid4().hex
            try:
                if self.type in (CERT_CLIENT, CERT_SERVER):
                    cache_db.set_add('openssl_tasks', task_id)
                    cache_db.publish('pooler', 'update')
                self._cert_request()
                self._generate_otp_secret()
                self.commit()
                self._cert_create()
            finally:
                if self.type in (CERT_CLIENT, CERT_SERVER):
                    cache_db.set_remove('openssl_tasks', task_id)
                    cache_db.publish('pooler', 'update')
            self._clean_openssl()

        if self.type == CERT_CLIENT_POOL:
            cache_db.set_add(
                self.org.get_cache_key('users_client_pool'), self.id)
        elif self.type == CERT_SERVER_POOL:
            cache_db.set_add(
                self.org.get_cache_key('users_server_pool'), self.id)
        elif self.type != CERT_CA:
            cache_db.set_add(self.org.get_cache_key('users'), self.id)
            self._add_cache_trie_key()

            self.org.queue_sort_users_cache(update_ip_pool=True)
            if self.type == CERT_CLIENT:
                LogEntry(message='Created new user "%s".' % self.name)

    def _add_cache_trie_key(self):
        users_trie = CacheTrie(self.org.get_cache_key('users_trie'))
        users_trie.add_key_terms(self.name, '%s-%s-%s' % (
            self.id, self.type, self.name))

    def _remove_cache_trie_key(self):
        users_trie = CacheTrie(self.org.get_cache_key('users_trie'))
        users_trie.remove_key_terms(self.name, '%s-%s-%s' % (
            self.id, self.type, self.name))

    def _setup_openssl(self):
        if not os.path.exists(self.temp_path):
            os.makedirs(self.temp_path)

        with open(self.index_path, 'a'):
            os.utime(self.index_path, None)

        with open(self.index_attr_path, 'a'):
            os.utime(self.index_attr_path, None)

        with open(self.serial_path, 'w') as serial_file:
            serial_file.write('01\n')

        with open(self.ssl_conf_path, 'w') as conf_file:
            conf_file.write(CERT_CONF % (
                self.org.id,
                self.org.path,
                self.temp_path,
                app_server.key_bits,
                self.id,
            ))

    def _clean_openssl(self):
        utils.rmtree(self.temp_path)

    def _cert_request(self):
        try:
            args = [
                'openssl', 'req', '-new', '-batch',
                '-config', self.ssl_conf_path,
                '-out', self.reqs_path,
                '-keyout', self.key_path,
                '-reqexts', '%s_req_ext' % self.type.replace('_pool', ''),
            ]
            subprocess.check_call(args, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            logger.exception('Failed to create user cert requests. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
            })
            raise
        os.chmod(self.key_path, 0600)

    def _cert_create(self):
        try:
            args = ['openssl', 'ca', '-batch']
            if self.type == CERT_CA:
                args += ['-selfsign']
            args += [
                '-config', self.ssl_conf_path,
                '-in', self.reqs_path,
                '-out', self.cert_path,
                '-extensions', '%s_ext' % self.type.replace('_pool', ''),
            ]
            subprocess.check_call(args, stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
        except subprocess.CalledProcessError:
            logger.exception('Failed to create user cert. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
            })
            raise

    def get_cache_key(self, suffix=None):
        if not self.cache_prefix:
            raise AttributeError('Cached config object requires cache_prefix')
        key = self.cache_prefix + '-' + self.org.id + '_' + self.id
        if suffix:
            key += '-%s' % suffix
        return key

    def load(self, *args, **kwargs):
        Config.load(self, *args, **kwargs)
        if not self.otp_secret:
            logger.debug('User otp secret missing generating new one. %r' % {
                self._generate_otp_secret()
            })

    def _generate_otp_secret(self):
        sha_hash = hashlib.sha512()
        sha_hash.update(os.urandom(8192))
        byte_hash = sha_hash.digest()
        for i in xrange(6):
            sha_hash = hashlib.sha512()
            sha_hash.update(byte_hash)
            byte_hash = sha_hash.digest()
        self.otp_secret = base64.b32encode(byte_hash)[:DEFAULT_OTP_SECRET_LEN]
        self.commit()

    def generate_otp_secret(self):
        self._generate_otp_secret()
        Event(type=USERS_UPDATED, resource_id=self.org.id)

    def verify_otp_code(self, code, remote_ip=None):
        if remote_ip:
            otp_cache = cache_db.get(self.get_cache_key('otp_cache'))
            if otp_cache:
                cur_code, cur_remote_ip = otp_cache.split(',')
                if cur_code == code and cur_remote_ip == remote_ip:
                    cache_db.expire(self.get_cache_key('otp_cache'),
                        OTP_CACHE_TTL)
                    return True
                else:
                    cache_db.remove(self.get_cache_key('otp_cache'))

        otp_secret = self.otp_secret
        padding = 8 - len(otp_secret) % 8
        if padding != 8:
            otp_secret = otp_secret.ljust(len(otp_secret) + padding, '=')
        otp_secret = base64.b32decode(otp_secret.upper())
        valid_codes = []
        epoch = int(time.time() / 30)
        for epoch_offset in range(-1, 2):
            value = struct.pack('>q', epoch + epoch_offset)
            hmac_hash = hmac.new(otp_secret, value, hashlib.sha1).digest()
            offset = ord(hmac_hash[-1]) & 0x0F
            truncated_hash = hmac_hash[offset:offset + 4]
            truncated_hash = struct.unpack('>L', truncated_hash)[0]
            truncated_hash &= 0x7FFFFFFF
            truncated_hash %= 1000000
            valid_codes.append('%06d' % truncated_hash)
        if code not in valid_codes:
            return False

        used_codes = cache_db.dict_get_all(self.get_cache_key('otp'))
        for auth_time, used_code in used_codes.items():
            if int(time.time()) - int(auth_time) > 120:
                cache_db.dict_remove(self.get_cache_key('otp'), auth_time)
            if used_code == code:
                return False

        cache_db.dict_set(self.get_cache_key('otp'),
            str(int(time.time())), code)
        cache_db.expire(self.get_cache_key('otp_cache'), OTP_CACHE_TTL)
        cache_db.set(self.get_cache_key('otp_cache'),
            ','.join((code, remote_ip)))
        return True

    def _get_key_info_str(self, user_name, org_name, server_name):
        return json.dumps({
            'version': CLIENT_CONF_VER,
            'user': user_name,
            'organization': org_name,
            'server': server_name,
        })

    def _build_key_archive(self):
        user_key_arcname = '%s_%s.key' % (self.org.name, self.name)
        user_cert_arcname = '%s_%s.crt' % (self.org.name, self.name)

        tar_file = tarfile.open(self.temp_key_archive_path, 'w')
        try:
            tar_file.add(self.key_path, arcname=user_key_arcname)
            tar_file.add(self.cert_path, arcname=user_cert_arcname)
            for server in self.org.iter_servers():
                server_cert_arcname = '%s_%s_%s.crt' % (
                    self.org.name, self.name, server.name)
                server_conf_path = os.path.join(self.org.path,
                    TEMP_DIR, '%s_%s.ovpn' % (self.id, server.id))
                server_conf_arcname = '%s_%s_%s.ovpn' % (
                    self.org.name, self.name, server.name)
                server.generate_ca_cert()
                tar_file.add(server.ca_cert_path, arcname=server_cert_arcname)

                client_conf = OVPN_CLIENT_CONF % (
                    self._get_key_info_str(
                        self.name, self.org.name, server.name),
                    server.protocol,
                    server.public_address, server.port,
                    server_cert_arcname,
                    user_cert_arcname,
                    user_key_arcname,
                )

                if server.otp_auth:
                    client_conf += 'auth-user-pass\n'

                with open(server_conf_path, 'w') as ovpn_conf:
                    ovpn_conf.write(client_conf)
                tar_file.add(server_conf_path, arcname=server_conf_arcname)
                os.remove(server_conf_path)
        finally:
            tar_file.close()

        return self.temp_key_archive_path

    def _build_inline_key_archive(self):
        tar_file = tarfile.open(self.temp_key_archive_path, 'w')
        try:
            for server in self.org.iter_servers():
                server_conf_path = os.path.join(self.org.path,
                    TEMP_DIR, '%s_%s.ovpn' % (self.id, server.id))
                server_conf_arcname = '%s_%s_%s.ovpn' % (
                    self.org.name, self.name, server.name)
                server.generate_ca_cert()

                client_conf = OVPN_INLINE_CLIENT_CONF % (
                    self._get_key_info_str(
                        self.name, self.org.name, server.name),
                    server.protocol,
                    server.public_address, server.port,
                )

                if server.otp_auth:
                    client_conf += 'auth-user-pass\n'

                client_conf += '<ca>\n%s\n</ca>\n' % utils.get_cert_block(
                    server.ca_cert_path)
                client_conf += '<cert>\n%s\n</cert>\n' % utils.get_cert_block(
                    self.cert_path)
                client_conf += '<key>\n%s\n</key>\n' % open(
                    self.key_path).read().strip()

                with open(server_conf_path, 'w') as ovpn_conf:
                    os.chmod(server_conf_path, 0600)
                    ovpn_conf.write(client_conf)
                tar_file.add(server_conf_path, arcname=server_conf_arcname)
                os.remove(server_conf_path)
        finally:
            tar_file.close()

        return self.temp_key_archive_path

    def build_key_archive(self):
        if app_server.inline_certs:
            return self._build_inline_key_archive()
        else:
            return self._build_key_archive()

    def clean_key_archive(self):
        try:
            os.remove(self.temp_key_archive_path)
        except OSError:
            pass

    def build_key_conf(self, server_id):
        server = self.org.get_server(server_id)
        conf_name = '%s_%s_%s.ovpn' % (self.org.name, self.name, server.name)
        server.generate_ca_cert()

        client_conf = OVPN_INLINE_CLIENT_CONF % (
            self._get_key_info_str(self.name, self.org.name, server.name),
            server.protocol,
            server.public_address, server.port,
        )

        if server.otp_auth:
            client_conf += 'auth-user-pass\n'

        client_conf += '<ca>\n%s\n</ca>\n' % utils.get_cert_block(
            server.ca_cert_path)
        client_conf += '<cert>\n%s\n</cert>\n' % utils.get_cert_block(
            self.cert_path)
        client_conf += '<key>\n%s\n</key>\n' % open(
            self.key_path).read().strip()

        return {
            'name': conf_name,
            'conf': client_conf,
        }

    def send_key_email(self, key_link_domain):
        email_from = persist_db.dict_get('auth', 'email_from')
        email_api_key = persist_db.dict_get('auth', 'email_api_key')

        if not email_from or not email_api_key:
            raise EmailNotConfiguredError('Email not configured', {
                'org_id': self.org.id,
                'user_id': self.id,
            })

        key_link = self.org.create_user_key_link(self.id)
        response = utils.request.post(POSTMARK_SERVER,
            headers={
                'Accept': 'application/json',
                'Content-Type': 'application/json',
                'X-Postmark-Server-Token': email_api_key,
            },
            json_data={
                'From': email_from,
                'To': self.email,
                'Subject': 'Pritunl VPN Key',
                'TextBody':  'Your vpn key can be downloaded from the ' +
                    'temporary link below. You may also directly import ' +
                    'your keys in the Pritunl client using the temporary ' +
                    'URI link.\n\n' +
                    'Key Link: ' + key_link_domain + key_link['view_url'] +
                    '\nURI Key Link: ' +
                    key_link_domain.replace('http', 'pt', 1) +
                    key_link['uri_url'],
            },
        )
        response = response.json()
        error_code = response.get('ErrorCode')
        error_msg = response.get('Message')

        if error_code == 0:
            pass
        elif error_code == 10:
            raise EmailApiKeyInvalid('Email api key invalid', {
                'org_id': self.org.id,
                'user_id': self.id,
                'error_code': error_code,
                'error_msg': error_msg,
            })
        elif error_code == 400:
            raise EmailFromInvalid('Email from invalid', {
                'org_id': self.org.id,
                'user_id': self.id,
                'error_code': error_code,
                'error_msg': error_msg,
            })
        else:
            logger.error('Unknown send user email error. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'error_code': error_code,
                'error_msg': error_msg,
            })
            raise EmailError('Unknown send user email error.', {
                'org_id': self.org.id,
                'user_id': self.id,
                'error_code': error_code,
                'error_msg': error_msg,
            })

    def clear_cache(self, org_data=True):
        if org_data:
            if self.type == CERT_CLIENT:
                cache_db.decrement(self.org.get_cache_key('user_count'))
            if self.type != CERT_CA:
                cache_db.set_remove(self.org.get_cache_key('users'), self.id)
                cache_db.list_remove(self.org.get_cache_key('users_sorted'),
                    self.id)
        if self.type in (CERT_CLIENT, CERT_SERVER):
            self._remove_cache_trie_key()
        cache_db.remove(self.get_cache_key('otp'))
        cache_db.remove(self.get_cache_key('otp_cache'))
        Config.clear_cache(self)

    def rename(self, name):
        self._remove_cache_trie_key()
        self.name = name
        self.commit()
        self._add_cache_trie_key()
        self.org.queue_sort_users_cache()
        Event(type=USERS_UPDATED, resource_id=self.org.id)

    def remove(self):
        self.clear_cache()
        name = self.name
        type = self.type
        self.org.queue_sort_users_cache(update_ip_pool=(type == CERT_CLIENT))

        try:
            os.remove(self.reqs_path)
        except OSError, error:
            logger.debug('Failed to remove user reqs file. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'path': self.reqs_path,
                'error': error,
            })

        try:
            os.remove(self.key_path)
        except OSError, error:
            logger.debug('Failed to remove user key file. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'path': self.reqs_path,
                'error': error,
            })

        try:
            os.remove(self.cert_path)
        except OSError, error:
            logger.debug('Failed to remove user cert file. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'path': self.reqs_path,
                'error': error,
            })

        try:
            os.remove(self.get_path())
        except OSError, error:
            logger.debug('Failed to remove user conf file. %r' % {
                'org_id': self.org.id,
                'user_id': self.id,
                'path': self.reqs_path,
                'error': error,
            })

        self._clean_openssl()

        Event(type=ORGS_UPDATED)
        Event(type=USERS_UPDATED, resource_id=self.org.id)
        if type == CERT_CLIENT:
            LogEntry(message='Deleted user "%s".' % name)

    @staticmethod
    def get_user(org, id):
        user = User(org, id=id)
        try:
            user.load()
        except IOError as exception:
            # File not found will happen when cached user list update is queued
            if exception.errno != 2:
                logger.exception('Failed to load user conf. %r' % {
                    'org_id': org.id,
                    'user_id': id,
                })
            return
        return user



from pritunl.constants import *
import pritunl.utils as utils
from pritunl.organization import Organization
from pritunl.server import Server
from pritunl import app_server, __version__

@app_server.app.route('/status', methods=['GET'])
@app_server.auth
def status_get():
    orgs_count = 0
    servers_count = 0
    servers_online_count = 0
    clients_count = 0

    for server in Server.iter_servers():
        servers_count += 1
        if server.status:
            servers_online_count += 1
        clients_count += len(server.clients)

    user_count = 0
    for org in Organization.iter_orgs():
        orgs_count += 1
        user_count += org.user_count

    local_networks = utils.get_local_networks()

    if app_server.openssl_heartbleed:
        notification = 'You are running an outdated version of openssl ' + \
            'containting the heartbleed bug. This could allow an attacker ' + \
            'to compromise your server. Please upgrade your openssl ' + \
            'package and restart the pritunl service.'
    else:
        notification = app_server.notification

    return utils.jsonify({
        'org_count': orgs_count,
        'users_online': clients_count,
        'user_count': user_count,
        'servers_online': servers_online_count,
        'server_count': servers_count,
        'server_version': __version__,
        'public_ip': app_server.public_ip,
        'local_networks': local_networks,
        'notification': notification,
    })



from constants import *
from cache import cache_db

class CacheObject:
    column_family = 'column_family'
    bool_columns = set()
    int_columns = set()
    float_columns = set()
    str_columns = set()
    cached_columns = set()
    db_instance = cache_db

    def __init__(self):
        self.all_columns = self.bool_columns | self.int_columns | \
            self.float_columns | self.str_columns
        self.id = None
        self._trans = None

    def __setattr__(self, name, value):
        if name != 'all_columns' and name in self.all_columns:
            if name in self.cached_columns:
                self.__dict__[name] = value

            if name in self.int_columns or name in self.float_columns:
                value = str(value)
            elif name in self.bool_columns:
                value = 't' if value else 'f'

            self.db.dict_set(self.get_cache_key(), name, value)
        else:
            self.__dict__[name] = value

    def __getattr__(self, name):
        if name in self.all_columns and self.id:
            if name in self.cached_columns and name in self.__dict__:
                return self.__dict__[name]

            value = self.db.dict_get(self.get_cache_key(), name)

            if name in self.int_columns:
                if value:
                    value = int(value)
            elif name in self.float_columns:
                if value:
                    value = float(value)
            elif name in self.bool_columns:
                if value == 't':
                    value = True
                elif value == 'f':
                    value = False

            if name in self.cached_columns:
                self.__dict__[name] = value

            return value
        elif name == 'db':
            return self._trans or self.db_instance
        raise AttributeError('Object instance has no attribute %r' % name)

    def transaction_start(self):
        self._trans = self.db.transaction()

    def transaction_commit(self):
        self._trans.commit()

    def get_cache_key(self, suffix=None):
        key = self.column_family + '-' + self.id
        if suffix:
            key += '-' + suffix
        return key

    def initialize(self):
        self.db.list_rpush(self.column_family, self.id)

    def remove(self):
        self.db.list_remove(self.column_family, self.id)
        self.db.remove(self.get_cache_key())

    @classmethod
    def iter_rows(cls):
        for row_id in cls.db_instance.list_iter(cls.column_family):
            row = cls(id=row_id)
            yield row

    @classmethod
    def get_last_row(cls):
        row_id = cls.db_instance.list_index(cls.column_family, -1)
        if row_id:
            return cls(id=row_id)
